%{
{$I SWITCHES.INC}
{ $Id: cmscan.l 1.4 1993/04/05 20:25:59 Ricki Exp $ }
unit cmscan;
{ command line handling }

{
  This code is in the public domain. You may use it freely.
  There are no guarantees, either expressed or implied,
  as to either merchantability or fitness for a particular
  purpose. The author's liability is limited to the amount
  you paid for it.

  Richard Breuer

  Brunssumstrasse 6
  5100 Aachen
  Germany

  Phone:  +49/241/85605
  Fax:    +49/241/8021329
  Email:  ricki@rwthi3.informatik.rwth-aachen.de
}

{
        $Log: cmscan.l $
        Revision 1.4  1993/04/05  20:25:59  Ricki
        address change

        Revision 1.3  1993/03/19  17:47:00  Ricki
        fixed a little syntax error

        Revision 1.2  1993/03/19  17:44:51  Ricki
        *** empty log message ***

        Revision 1.1  1993/01/21  18:52:16  Ricki
        Initial revision
}

interface

  uses
    strings,
    lexlib,
    dos;

  const
    version = '$Id: cmscan.l 1.4 1993/04/05 20:25:59 Ricki Exp $';
    maxcmcount = 64;

  type
    cmdstring = string[127]; { = comstr }

  function cmcount: byte;
  function cmstr(i: byte): cmdstring;
  function cmline: cmdstring;
    {
      cmcount   = paramcount
      cmstr     = paramstr
      cmline    returns the whole command line; note: leading and trailing
                whitespaces are trimmed
    }

implementation

  procedure addchar(c: char); forward;
  procedure endparam; forward;
  procedure addnum(n: string); forward;

  function input: char; forward;
  procedure unput(c: char); forward;

  var
    n: byte; { =cmcount+1 }
    start,len: array [1..maxcmcount] of byte;

    cmdptr: ^cmdstring;
    cmdstr,unquoted: cmdstring;
    p,l: byte;
%}

%start dflt str1 str2

whitespace                      [ \t\n]
digit                           [0-9]
number                          {digit}+

%%

!{number}                       addnum(yytext);
!'                              addchar('''');
!\"                             addchar('"');
!!                              addchar('!');
<dflt>{whitespace}+             endparam;
<dflt>'                         begin_(str1);
<dflt>\"                        begin_(str2);
<str1>'                         begin_(dflt);
<str1>''			addchar('''');
<str2>\"                        begin_(dflt);
<str2>\"\"                      addchar('"');

<dflt>.                         addchar(yytext[1]);
<str1>.                         addchar(yytext[1]);
<str2>.                         addchar(yytext[1]);

%%

  {local} procedure addchar(c: char);
  { appends c to unquoted and increments the current length }
  begin
    unquoted := unquoted+c;
    inc(l)
  end;

  {local} procedure endparam;
  { ends construction of the current string }
  begin
    if (n>0) then begin
      start[n] := length(unquoted)-l+1;
      len[n] := l
    end;
    inc(n); l := 0
  end;

  {local} procedure addnum(n: string);
  { adds the character whose code is in n to unquoted }
  var
    z: longint;
    b: array [0..3] of char absolute z;
    code: integer;
  begin
    val(copy(n,2,255),z,code);
    if (code<>0) then z := 0;
    { add from the most significant non-null byte }
    if (b[3]=#0) then begin
      if (b[2]=#0) then begin
        if (b[1]=#0) then addchar(b[0])
        else begin
          addchar(b[1]); addchar(b[0])
        end
      end else begin
        addchar(b[2]); addchar(b[1]); addchar(b[0])
      end
    end else begin
      addchar(b[3]); addchar(b[2]); addchar(b[1]); addchar(b[0])
    end
  end;

  {local} function input: char;
  { redefinition for lex: read from cmdstr, not stdin }
  begin
    inc(p);
    if (p>length(cmdstr)) then input := #0
                          else input := cmdstr[p]
  end;

  {local} procedure unput(c: char);
  { redefinition for lex: go back one character in cmdstr }
  begin if (p>0) then dec(p) end;

  function cmcount: byte;
  begin cmcount := n-1 end;

  function cmstr(i: byte): cmdstring;
  begin
    if (i=0) then
      cmstr := paramstr(0)
    else if (i>0) and (i<=n) then
      cmstr := copy(unquoted,start[i],len[i])
    else
      cmstr := ''
  end;

  function cmline;
  begin cmline := cmdstr end;

begin
  { get the whole command line and remove leading and trailing whitespace }
  cmdptr := ptr(prefixseg,$80); cmdstr := cmdptr^;
  cmdstr := headtrim(cmdstr,whitespace);
  cmdstr := tailtrim(cmdstr,whitespace);

  { initialization }
  unquoted := '';
  n := 1;
  l := 0;
  p := 0;
  begin_(dflt);

  if (cmdstr<>'') then begin
    { parse the command line }
    if (yylex=0) then {};

    { terminate the last parameter }
    endparam
  end
end.
