Date: Tue, 29 Dec 1992 20:28:00 CET
From: a.gorup@amazonas.zer.sub.org
Reply-To: a.gorup@amazonas.zer.sub.org
Distribution: zer
Message-ID: <62.6147@AMAZONAS.zer.sub.org>
Newsgroups: zer.z-netz.rechner.ibm.programmieren
Subject: Julianisches Datum
Organization: Z-NETZ BBS Network, FRG
Content-Length: 3917    
Lines: 134     
X-Gateway: NETCALL3.8 UU uzercp.zer.sub.org [UZERCP V3.99c]

@ ETranfer aborted by otherside.*
 
 
 
 
 
UNIT JULIAN;
 
(*===========================================================================*)
(*=                                                                         =*)
(*=  ACM ALGORITHM 199                                                      =*)
(*=  CONVERSIONS BETWEEN CALENDAR DATE AND JULIAN DAY NUMBER                =*)
(*=  Robert G. Tantzen                                                      =*)
(*=  Air Force Missile Development Center, Holloman AFB, New Mex.           =*)
(*=                                                                         =*)
(*=  ACM Communications 8 (Aug. 1963),444.                                  =*)
(*=                                                                         =*)
(*===========================================================================*)
 
INTERFACE
 
  PROCEDURE JDAY(d,m,y:longint; var jd:longint);
 
    { JDAY converts a calendar date, Gregorian calendar, to the corresponding
      Julian day number j. From the given day d, month m, and year y, the
      Julian day number j is computed without using tables. The procedure is
      valid for any valid Gregorian calendar date. When transcribing JDAY for
      other compilers, be sure that integers of size 3*10^^6 can be handled
    }
 
  PROCEDURE JDATE(j:longint; var d,m,y:longint);
 
    { JDATE converts a Julian day number j to the corresponding calendar date,
      Gregorian calendar. Since j is an integer for this procedure, it is
      correct astronomically for noon of the day. JDATE computes the day d,
      month m, and year y, without using tables. The procedure is valid for
      any valid Gregorian calendar date. When transcribing JDATE for other
      compilers, be sure that integers of size 3*10^^6 can be handled
    }
 
 
  PROCEDURE KDAY(d,m,ya:longint; var k:longint);
 
    { KDAY converts a calendar date, GREGORIAN calendar, to the corresponding
      serial day number k. From the given day d, month m, and the last two
      decimals of the year, ya, the serial day number k is computed without
      using tables. The procedure is valid from 1 March 1900 (k=1) to 31 De-
      cember 1999 (=36465). To obtain the Julian day number j (valid at noon)
      use j=k+2415079.
    }
 
  PROCEDURE KDATE(k:longint; var d,m,ya:longint);
 
    { KDATE converts a serial day number k to the corresponding calendar date,
      Gregorian calendar. It computes day d, month m, and the last two decimals
      of the year, ya, without using tables. The procedure is valid from k=1
      (1 March 00) to k=36465 (31 December 99) for any one century. For the
      20th Century the relation between k and the Julian day number j (at noon)
      is j=k+2415079
    }
 
IMPLEMENTATION
 
  PROCEDURE JDAY;
 
    var c,ya:longint;
 
    begin
      if m>2 then m:=m-3
      else begin
        m:=m+9;
        y:=y-1;
      end;
      c:=y div 100;
      ya:=y-100*c;
      jd:=(146097*c) div 4 + (1461*ya) div 4 +
          (153*m+2) div 5 + d + 1721119;
    end;
 
 
 
  PROCEDURE JDATE;
 
    begin
      j:=j-1721119;
      y:=(4*j-1) div 146097;
      j:=4*j-1-146097*y;
      d:=j div 4;
      j:=(4*d+3) div 1461;
      d:=4*d+3-1461*j;
      d:=(d+4) div 4;
      m:=(5*d-3) div 153;
      d:=5*d-3-153*m;
      d:=(d+5) div 5;
      y:=100*y+j;
      if m<10 then m:=m+3
      else begin
        m:=m-9;
        y:=y+1;
      end;
    end;
 
  
  PROCEDURE KDAY;
 
    begin
      if m>2 then m:=m-3
      else begin
        m:=m+9;
        ya:=ya-1;
      end;
      k:=(1461*ya) div 4 + (153*m+2) div 5 + d;
    end;
 
 
  PROCEDURE KDATE;
 
    begin
      ya:=(4*k-1) div 1461;
      d:=4*k-1-1461*ya;
      d:=(d+4) div 4;
      m:=(5*d-3) div 153;
      d:=5*d-3-153*m;
      d:=(d+5) div 5;
      if m<10 then m:=m+3
      else begin
        m:=m-9;
        ya:=ya+1;
      end;
    end;
 
END. { JULIAN }

