{$I SWITCHES.INC}
{ $Id: dostools.pas 1.4 1993/04/05 19:59:55 Ricki Exp $ }
Unit DosTools;
{ DOS-specific utility procedures }

{
  This code is in the public domain. You may use it freely.
  There are no guarantees, either expressed or implied,
  as to either merchantability or fitness for a particular
  purpose. The author's liability is limited to the amount
  you paid for it.

  Richard Breuer

  Brunssumstrasse 6
  5100 Aachen
  Germany

  Phone:  +49/241/85605
  Fax:    +49/241/8021329
  Email:  ricki@pool.informatik.rwth-aachen.de


  This unit is based on the unit DosTools, which can be found
  in the DRCPAS10 package by

  David R. Conrad, 17 Nov 92
  David_Conrad@mts.cc.wayne.edu
  dave@michigan.com

  at SIMTEL in the directory msdos/turbopas. Some procedures are
  from the unit Tools in the same package.
}

{
        $Log: dostools.pas $
        Revision 1.4  1993/04/05  19:59:55  Ricki
        address change

        Revision 1.3  1993/03/27  15:39:58  Ricki
        new procedure ResetStdin

        Revision 1.2  1993/03/19  17:44:09  Ricki
        new procedures setfileatt, getfileatt

        Revision 1.1  1993/01/22  00:55:37  Ricki
        Initial revision
}

Interface

  Uses
    tsuntj,
    Dos;

  const
    pversion = '$Id: dostools.pas 1.4 1993/04/05 19:59:55 Ricki Exp $';

    { for use with Boot, to select boot type }
    WarmBoot  = 0;
    ColdBoot  = 1;
    ResetBoot = 2;

  Procedure Boot(BootType: byte);
  Procedure Call(Proc: Pointer);
  Procedure DisableInterrupts; Inline($FA); {CLI}
  Procedure EnableInterrupts;  Inline($FB); {STI}
  Procedure InitRegisters(var Reg: Registers);
  Procedure DecPtr(Var P: Pointer);
  Procedure DecFarPtr(Var P: Pointer);
  Procedure IncPtr(Var P: Pointer);
  Procedure IncFarPtr(Var P: Pointer);
  Function InpRedirected: Boolean;
  Function OutRedirected: Boolean;
  Function OutToNUL: Boolean;
  procedure resetstdin(var f: text);
  procedure SetFileAtt(fn: string; attr: word);
  procedure GetFileAtt(fn: string; var attr: word; var ok: boolean);
    {
      boot          reboot the computer, flushing caches first
      call          call a procedure with no parameters
      InitRegisters initialize variable of type registers: slightly
                    anal-retentive [??? - Ricki]
      DecPtr        decrement a pointer, Note: Will wrap around the
                    beginning of segments!
      DecFarPtr     decrement a pointer, handling segments
      IncPtr        increment a pointer, Note: Will wrap around the
                    end of segments!
      IncFarPtr     increment a pointer, handling segments
      inpRedirected returns whether standard input has been redirected,
                    ie. PROG < FILE or OTHER | PROG
      outRedirected returns whether standard output has been redirected,
                    ie. PROG > FILE
      outToNUL      returns whether standard output has been redirected
                    to NUL, ie. PROG > NUL
      ResetStdIn    resets standard input w/seek to position 0
      SetFileAtt    similar to SetFAttr, but good for dirs, too
      GetFileAtt    similar to GetFAttr, but good for dirs, too
    }

Implementation

  Procedure Boot(BootType: byte);
  const
    bootwarm = $1234;
    bootcold = 0;
  var
    reg: registers;
    bootaddr: pointer;
    warmcold: word absolute $40:$72;
    shifts: byte absolute $40:$17;
  begin
    initregisters(reg);
    bootaddr := ptr($FFFF,0);
    case boottype of
      warmboot: warmcold := bootwarm;
      coldboot: warmcold := bootcold;
      else exit
    end;
    reg.ah := $0D;
    intr($21,reg);
    shifts := (4 OR 8);                  {Ctrl-Alt}
    repeat
      reg.flags := FCarry;
      reg.ax := $4F53;                   {Translate scan codes, Del}
      intr($15,reg);
    until (reg.flags AND FCarry) = FCarry;
    case boottype of
      warmboot : call(bootaddr);         {One call we won't be returning from}
      coldboot : call(bootaddr);
      resetboot: begin
                    DisableInterrupts;
                    port[$64] := $FE;    {Tell keyboard to reset processor}
                    while (true) do;     {Wait for it}
                  end;
    end;
  end;

  var
    CallAddr: Pointer;

  Procedure Call(Proc: Pointer);
    Procedure CallProc; Inline($FF/$1E/CallAddr); {CALL DWORD PTR CallAddr}
  begin
    CallAddr := Proc;
    CallProc;
  end;

  Procedure InitRegisters(var Reg: Registers);
  begin
    fillchar(reg,sizeof(reg),0);
    reg.ds := dseg;
    reg.es := dseg;
  end;

  Procedure IncPtr (Var P : Pointer);
  begin
    P := Ptr(Seg(P^),Succ(Ofs(P^)));
  end;

  Procedure IncFarPtr (Var P : Pointer);
  begin
    If Succ(Ofs(P^)) = 0 then
      P := Ptr(Seg(P^)+$1000,0)
    Else
      P := Ptr(Seg(P^),Succ(Ofs(P^)));
  end;

  Procedure DecPtr (Var P : Pointer);
  begin
    P := Ptr(Seg(P^),Pred(Ofs(P^)));
  end;

  Procedure DecFarPtr (Var P : Pointer);
  begin
    If Ofs(P^) = 0 then
      P := Ptr(Pred(Seg(P^)),15)
    Else
      P := Ptr(Seg(P^),Pred(Ofs(P^)));
  end;

  Function InpRedirected: Boolean;
  begin inpredirected := pipedifn end;

  Function OutRedirected: Boolean;
  begin outredirected := pipedofn end;

  Function OutToNUL: Boolean;
  begin outtonul := pipednfn end;

  procedure resetstdin(var f: text);
  {
    ----------2142-------------------------------
    INT 21 - DOS 2+ - "LSEEK" - SET CURRENT FILE POSITION
	    AH = 42h
	    AL = origin of move
	        00h start of file
	        01h current file position
	        02h end of file
	    BX = file handle
	    CX:DX = offset from origin of new file position
    Return: CF clear if successful
	        DX:AX = new file position in bytes from start of file
	    CF set on error
	        AX = error code (01h,06h) (see AH=59h)
    Notes:	for origins 01h and 02h, the pointer may be positioned before the
	      start of the file; no error is returned in that case, but subsequent
	      attempts at I/O will produce errors
	    if the new position is beyond the current end of file, the file will
	      be extended by the next write (see AH=40h)
    SeeAlso: AH=24h,INT 2F/AX=1228h
  }
  var
    regs : Registers;
  begin
    { this idea is from
        Edwin Schwartz
        Niederzissener Strasse 12
        5479 Wehr }
    reset(f);
    regs.ax := $4200;
    regs.bx := 0;
    regs.cx := 0;
    regs.dx := 0;
    intr($21,regs)
  end;

  procedure SetFileAtt(fn: string; attr: word);
  {
    ----------214301-----------------------------
    INT 21 - DOS 2+ - "CHMOD" - SET FILE ATTRIBUTES
	    AX = 4301h
	    CX = new attributes
	        bit 8: shareable (Novell NetWare)
		    7: unused
		    6: unused
		    5: archive
		    4: directory
		    3: volume label
		    2: system
		    1: hidden
		    0: read-only
	    DS:DX -> ASCIZ filename
    Return: CF clear if successful
	        AX destroyed
	    CF set on error
	        AX = error code (01h,02h,03h,05h) (see AH=59h)
    Notes:	will not change volume label or directory attributes
	    MSDOS 4.01 reportedly closes the file if it is currently open
    SeeAlso: AX=4300h,INT 2F/AX=110Eh
  }
  var
    regs : Registers;
    r: array [1..sizeof(registers)] of byte absolute regs;
  begin
    fn := fn+#0;	{ ASCIIZ }

    regs.ah := $43;
    regs.al := $01;
    regs.ch := $00;
    regs.cl := attr and (not directory) and (not volumeid);
    regs.dx := ofs(fn[1]);
    regs.ds := seg(fn);
    intr($21,regs)
  end;

  procedure GetFileAtt(fn: string; var attr: word; var ok: boolean);
  {
    ----------214300-----------------------------
    INT 21 - DOS 2+ - GET FILE ATTRIBUTES
	    AX = 4300h
	    DS:DX -> ASCIZ filename
    Return: CF clear if successful
	        CX = attributes (see AX=4301h)
	        AX = CX (DR-DOS 5.0)
	    CF set on error
	        AX = error code (01h,02h,03h,05h) (see AH=59h)
    SeeAlso: AX=4301h,AH=B6h,INT 2F/AX=110Fh
  }
  var
    regs : Registers;
  begin
    fn := fn+#0;	{ ASCIIZ }

    regs.ah := $43;
    regs.al := $00;
    regs.dx := ofs(fn[1]);
    regs.ds := seg(fn);
    intr($21,regs);

    attr := regs.cx;

    ok := (regs.cx and $f000=0)
  end;

End.
