{$I SWITCHES.INC}
{ $Id: files.pas 1.3 1993/04/05 19:59:42 Ricki Exp $ }
unit files;
{ various file and filename handling procedures }

{
  This code is in the public domain. You may use it freely.
  There are no guarantees, either expressed or implied,
  as to either merchantability or fitness for a particular
  purpose.  The author's liability is limited to the amount
  you paid for it.

  Richard Breuer

  Brunssumstrasse 6
  5100 Aachen
  Germany

  Phone:  +49/241/85605
  Fax:    +49/241/8021329
  Email:  ricki@pool.informatik.rwth-aachen.de


  The procedures forall, forallrec, gooddir, shrinkpath, and errstr
  are derived from Borland's Turbo Technix Sept/Oct 1988. These programs
  can be found at SIMTEL in the directory msdos/turbopas as SPOC88.ARC.
}

{
        $Log: files.pas $
        Revision 1.3  1993/04/05  19:59:42  Ricki
        address change

        Revision 1.2  1993/03/19  17:38:44  Ricki
        some internal modifications
        new functions fillname, attr2str

        Revision 1.1  1993/01/22  00:55:20  Ricki
        Initial revision
}

interface

  uses
    standard,
    strings,
    numbers,
    tsuntb,tsuntg,tsuntj,tsunte,
    dos;

  const
    pversion = '$Id: files.pas 1.3 1993/04/05 19:59:42 Ricki Exp $';

  type
    namexstr = string[8+1+3]; { xxxxxxxx.ext }
    attrstr  = string[4];     { arhs }
    TfileProc = procedure(var s: searchrec; p: pathstr); { for forall[rec] }

  procedure forall(mask: pathstr; attr: byte; proc: TfileProc;
                   var code: byte);
  procedure forallrec(path: pathstr; mask: namexstr; attr: byte;
                      proc: TfileProc; var code: byte);
  procedure forlistfile(name: pathstr; attr: byte; proc: TfileProc;
                        var code: byte; var codeline: longint);
  function gooddir(s: searchrec): boolean;
  procedure shrinkpath(var path: pathstr);
  function FileErrStr(errcode: byte): string;
    {
      forall      for all files mask with attr in the current directory do proc
      forallrec   for all files mask with attr under path do proc recursivly
      forlistfile read names from a listfile and perform a forall on them
      gooddir     does s contain a directory other than . or ..?
      shrinkpath  extracts dirstr from path
      FileErrStr  returns a message identifying an error code
                   2: File not found
                   3: Path not found
                   5: Access denied
                   6: Invalid handle
                   8: Not enough memory
                  10: Invalid environment
                  11: Invalid format
    }

  function tmpname(dir: pathstr): pathstr;
  function validname(n: namexstr): boolean;
    {
      tmpname   create a unique file name in the directory dir
      validname is n a valid DOS file name?
    }

  function filename(path: pathstr): namestr;
  function filedir(path: pathstr): dirstr;
  function fileext(path: pathstr): extstr;
  function filenamex(path: pathstr): namexstr;
  function fillname(path: pathstr): string;
    {
      filename  extracts the name from path
      filedir   extracts the dir from path
      fileext   extracts the extension from path
      filenamex extracts name+ext from path
      fillname  fills path up with spaces up to the full name length,
                eg, "C:\TEST" becomes "C:\TEST        ", with the length
                of the longest possible name "C:\TESTxxxx.xxx"
    }

  function exists(n: pathstr): boolean;
  function opened(var f: text): boolean;
  function isdir(n: pathstr): boolean;
  function maybemany(n: pathstr): boolean;
    {
      exists    returns whether n exists
      opened    returns whether f is currently opened
      isdir     returns whether n is a directory
      maybemany returns whether n represents (maybe) more than 1 file,
                ie., whether it is a directory or contains wildcards
    }

  function numfloppies: byte;
  function firstdrive: char;
  function drivein(d: char): boolean;
  function drivefixed(d: char): boolean;
    {
      numfloppies the number of floppy drives
      firstdrive  the character of the first drive
      drivein     is a disk in drive d?
      drivefixed  is d a fixed drive or a floppy?
    }

  procedure diskinfo(drive: char; var vol: namexstr; var size,free: longint;
                     var code: word);
  procedure fileinfo(name: string; var size,time: longint;
                     var attr: word; var code: word);
    {
      diskinfo  extracts useful information about a diskette
      fileinfo  dito for a file
    }

  function attr2str(attr: word): attrstr;
    {
      attr2str  returns a string "arhs" which represents the attributes
                Archive, ReadOnly, Hidden, and System (-, if not set)
    }

implementation

  var
    pdir: dirstr;
    pnam: namestr;
    pext: extstr;

    enginemask: namexstr;
    engineattr: byte;
    engineproc: TfileProc;
    enginecode: byte;

  procedure forall(mask: pathstr; attr: byte; proc: TfileProc;
                   var code: byte);
  var
    s: searchrec;
    p: pathstr;
    ext: extstr;
  begin
    fsplit(mask,p,mask,ext);
    mask := mask+ext;
    findfirst(p+mask,attr,s);
    if (doserror<>0) then begin code := doserror; exit end;
    while (doserror=0) do begin
      proc(s,p);
      findnext(s)
    end;
    if (doserror=18) then code := 0
                     else code := doserror
  end;

  function gooddir(s: searchrec): boolean;
  begin
    gooddir := (s.name<>'.') and
               (s.name<>'..') and
               (s.attr and directory=directory)
  end;

  procedure shrinkpath(var path: pathstr);
  var
    p: byte;
    dummy: namestr;
  begin
    fsplit(path,path,dummy,dummy);
    dec(path[0])
  end;

  {local}
  {$f+} procedure searchonedir(var s: searchrec; p: pathstr); {$f-}
  {recursive procedure to search one directory}
  begin
    if (gooddir(s)) then begin
      p := p+s.name;
      forall(p+'\'+enginemask,engineattr,engineproc,enginecode);
      forall(p+'\*.*',directory or archive,searchonedir,enginecode)
    end;
  end;

  procedure forallrec(path: pathstr; mask: namexstr; attr: byte;
                      proc: TfileProc; var code: byte);
  begin
    { set up unit global variables for use in recursive directory
      search procedure }
    enginemask := mask;
    engineproc := proc;
    engineattr := attr;
    enginecode := 0;
    { adjust path, if necessary }
    if (path='') then path := '.\';
    if (path[length(path)]<>'\') then path := path+'\';
    { current dir & recursion }
    forall(path+mask,attr,proc,code);
    forall(path+'*.*',directory or attr,searchonedir,code);
    code := enginecode
  end;

{$IFDEF DEBUG}{$i-}{$ENDIF}
  procedure forlistfile(name: pathstr; attr: byte; proc: TfileProc;
                        var code: byte; var codeline: longint);
  var
    f: text;
    p: byte;
    line: string;
    dummy: longint;
  begin
    codeline := 0;
    assign(f,name); reset(f);
    code := lo(ioresult);
    if (code<>0) then exit;
    { loop over the filenames in f }
    while (not eof(f)) do begin
      { read one line }
      readln(f,line);
      inc(codeline);
      { kill leading and trailing whitespace }
      line := headtrim(line,whitespace);
      line := tailtrim(line,whitespace);
      { loop over the names in line, delimited by whitespace }
      while (line<>'') do begin
        p := minpos(whitespace,line);
        if (p=0) then p := length(line)+1;
        { recurse, if the name is again a listfile name }
        if (line[1]='@')
          then forlistfile(copy(line,2,p-2),attr,proc,code,dummy)
          else forall(copy(line,1,p-1),attr,proc,code);
        if (code<>0) then exit;
        delete(line,1,p);
      end
    end
  end;
{$IFDEF DEBUG}{$i+}{$ENDIF}

  function FileErrStr(errcode: byte): string;
  begin
    case errcode of
       0: FileErrStr  := '';
       2: FileErrStr  := 'File not found';
       3: FileErrStr  := 'Path not found';
       5: FileErrStr  := 'Access denied';
       6: FileErrStr  := 'Invalid handle';
       8: FileErrStr  := 'Not enough memory';
      10: FileErrStr  := 'Invalid environment';
      11: FileErrStr  := 'Invalid format';
      18: FileErrStr  := 'No more files';
      else FileErrStr := 'Error #'+numstr(errcode)
    end
  end;

{$IFDEF DEBUG}{$i-}{$ENDIF}
  function tmpname(dir: pathstr): pathstr;
  const
    maxtries = 10;
  var
    i,tries: integer;
    s: namexstr;
    f: file;
    ok: boolean;
  begin
    tries := 0; s := '00000000.TMP';
    { append a \ to dir, if necessary }
    if (dir<>'') then
      if (dir[length(dir)]<>'\') then dir := dir+'\';
    { try for at most maxtries times to create a unique filename }
    repeat
      inc(tries);
      randomize;
      for i:=1 to 8 do s[i] := chr(random(9)+ord('0'));
      assign(f,dir+s); reset(f,1);
      ok := (ioresult=2); { file not found }
      if (not ok) then begin
        close(f);
        { exit if somethings wrong }
        if (ioresult<>0) then tries := maxtries
      end
    until (ok) or (tries=maxtries);
    if (not ok) then err('Cannot create unique name');
    tmpname := dir+s
  end;
{$IFDEF DEBUG}{$i+}{$ENDIF}

  function validname(n: namexstr): boolean;
  var
    i: byte;
  begin
    for i:=1 to length(n) do begin
      if ( (n[i]>#0) and (n[i]<#33) )
      or ( (n[i]=#34) )
      or ( (n[i]>#41) and (n[i]<#46) )
      or ( (n[i]=#47) )
      or ( (n[i]>#57) and (n[i]<#64) )
      or ( (n[i]>#90) and (n[i]<#94) )
      or ( (n[i]=#124) )
      then begin validname := false; exit end
    end;
    validname := (length(n)>0)
  end;

  function filename(path: pathstr): namestr;
  begin
    fsplit(path,pdir,pnam,pext); filename := strupcase(pnam)
  end;

  function filedir(path: pathstr): dirstr;
  begin
    fsplit(path,pdir,pnam,pext); filedir := strupcase(pdir)
  end;

  function fileext(path: pathstr): extstr;
  begin
    fsplit(path,pdir,pnam,pext); fileext := strupcase(pext)
  end;

  function filenamex(path: pathstr): namexstr;
  begin
    fsplit(path,pdir,pnam,pext); filenamex := strupcase(pnam+pext)
  end;

  function fillname(path: pathstr): string;
  begin
    fsplit(path,pdir,pnam,pext);
    fillname := strupcase(pdir+copy(pnam+pext+blanks,1,13))
  end;

  function exists(n: pathstr): boolean;
  begin exists := fexistfn(n) end;

  function opened(var f: text): boolean;
  begin opened := openedfn(f) end;

  function numfloppies: byte;
  begin numfloppies := drivesfn end;

  function firstdrive: char;
  begin firstdrive := fdrivefn end;

  function drivein(d: char): boolean;
  begin drivein := indrivfn(d) end;

  function drivefixed(d: char): boolean;
  begin drivefixed := fixedfn(d) end;

  function isdir(n: pathstr): boolean;
  begin
    if (length(n)=2) and (n[2]=':') then isdir := isdirfn(n+'.')
    				    else isdir := isdirfn(n)
  end;

  function maybemany(n: pathstr): boolean;
  begin
    maybemany := (isdir(n)) or (pos('?',n)>0)
                            or (pos('*',n)>0)
  end;

  procedure diskinfo(drive: char; var vol: namexstr; var size,free: longint;
                     var code: word);
  var
    sr: searchrec;
    db: byte;
  begin
    vol := '';
    size := 0;
    free := 0;
    code := 0;
    drive := upcase(drive);
    if (drive<'A') or (drive>'Z') then begin code := 65535; exit end;
    if (doserror<>0) then ;

    { label }
    FindFirst(drive+':\*.*',Volumeid,sr);
    if (doserror=0) then begin
      vol := sr.name;
      substone('.','',vol)
    end;

    { size & free space }
    db := ord(drive)+1-ord('A');
    free := diskfree(db);
    if (code=0) then code := doserror;

    size := disksize(db);
    if (code=0) then code := doserror
  end;

  procedure fileinfo(name: string; var size,time: longint;
                     var attr: word; var code: word);
  var
    sr: searchrec;
  begin
    findfirst(name,$ff,sr);
    code := doserror;
    if (code<>0) then begin
      size := 0;
      time := 0;
      attr := 0
    end else begin
      size := sr.size;
      time := sr.time;
      attr := sr.attr
    end
  end;

  function attr2str(attr: word): attrstr;
  begin
    attr2str[0] := #4;
    if (attr and archive=archive) then attr2str[1] := 'a'
                                  else attr2str[1] := '-';
    if (attr and readonly=readonly) then attr2str[2] := 'r'
                                    else attr2str[2] := '-';
    if (attr and hidden=hidden) then attr2str[3] := 'h'
                                else attr2str[3] := '-';
    if (attr and sysfile=sysfile) then attr2str[4] := 's'
                                  else attr2str[4] := '-';
  end;

end.
