{$I SWITCHES.INC}
{ $Id: math.pas 1.2 1993/04/05 20:01:04 Ricki Exp $ }
unit math;
{ various math procedures }

{
  This code is in the public domain. You may use it freely.
  There are no guarantees, either expressed or implied,
  as to either merchantability or fitness for a particular
  purpose.  The author's liability is limited to the amount
  you paid for it.

  Richard Breuer

  Brunssumstrasse 6
  5100 Aachen
  Germany

  Phone:  +49/241/85605
  Fax:    +49/241/8021329
  Email:  ricki@pool.informatik.rwth-aachen.de
}

{
        $Log: math.pas $
        Revision 1.2  1993/04/05  20:01:04  Ricki
        address change

        Revision 1.1  1993/03/20  08:25:30  Ricki
        Initial revision
}

interface

  const
    pversion = '$Id: math.pas 1.2 1993/04/05 20:01:04 Ricki Exp $';
    eps = 0.00001;

  function eq(a,b: real): boolean;
  function pot(a,b: real): real;
  function ipot(a: real; b: integer): real;
  function ArcCos(x: real): real;
  function ArcSin(x: real): real;
  function tan(x: real): real;
  {
    eq 		determines if two reals are 'equal'
    pot		calc a^b for a>=0
    ipot	calc a^b for all a
    arccos	for |x|<=1
    arcsin	for |x|<=1
    tan		for x<>(2k+1)*(pi/2)
  }

implementation

  function eq(a,b: real): boolean;
  begin eq := (abs(a-b)<eps) end;

  function pot(a,b: real): real;
  begin
    if (a<eps) then pot := 0.0
    else            pot := exp(b*ln(a))
  end;

  function ipot(a: real; b: integer): real;
  var
    r: real;
  begin
    if (b<0) then ipot := 1.0/ipot(a,-b)
    else begin
      r := 1.0;
      while (b>0) do if (odd(b)) then begin
        r := r*a;
	dec(b)
      end else begin
        a := sqr(a);
        b := b shr 1
      end;
      ipot := r
    end
  end;

  function ArcCos (x:real) : real;
  begin
    ArcCos := Pi/2 - ArcTan (x/Sqrt(1-x*x));
  end;  {ArcCos}

  function ArcSin (x:real) : real;
  begin
    ArcSin := ArcTan (x/Sqrt(1-x*x));
  end;  {ArcSin}

  function Tan(x :real): real;
  begin Tan := Sin(x)/Cos(x)
  end;  {Tan}

end.
