{$I SWITCHES.INC}
{ $Id: numbers.pas 1.3 1993/04/05 19:59:11 Ricki Exp $ }
unit numbers;
{ various procedures for number conversion and number formatting }

{
  This code is in the public domain. You may use it freely.
  There are no guarantees, either expressed or implied,
  as to either merchantability or fitness for a particular
  purpose. The author's liability is limited to the amount
  you paid for it.

  Richard Breuer

  Brunssumstrasse 6
  5100 Aachen
  Germany

  Phone:  +49/241/85605
  Fax:    +49/241/8021329
  Email:  ricki@pool.informatik.rwth-aachen.de


  Many procedures in this unit are derived from the unit Tools,
  which can be found in the DRCPAS10 package by

  David R. Conrad, 17 Nov 92
  David_Conrad@mts.cc.wayne.edu
  dave@michigan.com

  at SIMTEL in the directory msdos/turbopas.
}

{
        $Log: numbers.pas $
        Revision 1.3  1993/04/05  19:59:11  Ricki
        address change

        Revision 1.2  1993/03/19  17:32:37  Ricki
        new functions hval, oval, cnumval

        Revision 1.1  1993/01/22  00:55:32  Ricki
        Initial revision
}

interface

  const
    pversion = '$Id: numbers.pas 1.3 1993/04/05 19:59:11 Ricki Exp $';

  {
    number conversion
  }
  function bcdbin(b: byte): byte;
  function binbcd(b: byte): byte;
  function binb(b: byte): string;
  function binw(w: word): string;
  function binl(l: longint): string;
  function hexit(h: byte): char;
  function hexb(b: byte): string;
  function hexw(w: word): string;
  function hexl(l: longint): string;
  function octb(b: byte): string;
  function octw(w: word): string;
  function octl(l: longint): string;
    {
      bcdbin converts a binary coded decimal to binary
      binbcd converts a byte in the range 0..99 to binary coded decimal

      binb converts a byte to a string of binary digits
      binw converts a word to binary
      binl converts a longint to binary

      hexit converts a byte in the range 0..15 to hexadecimal '0'..'F'
      hexb  converts a byte to hexadecimal
      hexw  converts a word to hexadecimal
      hexl  converts a longint to hexadecimal

      octb converts a byte to octal
      octw converts a word to octal
      obtl converts a longint to octal
    }

  {
    number formatting
  }
  function commastr(l: longint): string;
  function commarstr(r: real; d: byte): string;
  function lnumstr(n: longint; w: byte): string;
  function lnumrstr(n: real; w,d: byte): string;
  function numstr(n: longint): string;
  function numrstr(n: real): string;
  function zerostr(l: longint; w: byte): string;
  function zerorstr(r: real; w,d: byte): string;
    {
      commastr  formats an integer with commas (eg. 1,234,567)
      commarstr formats a real with commas

      lnumstr   converts an integer to a string, right justified in w spaces
      lnumrstr  converts a real in w spaces, d decimal places
      numstr    like lnumstr, best fit format
      numrstr   like lnumrstr, best fit format

      zerostr   formats an integer right justified in a field of zeroes
      zerorstr  formats a real with d decimal places in w zeroes
    }

  {
    miscellanous procedures
  }
  function ptrstr(p: pointer): string;
  function ival(s: string; var i: longint): boolean;
  function hval(s: string; var i: longint): boolean;
  function oval(s: string; var i: longint): boolean;
  function cnumval(s: string; var i: longint): boolean;
  function roman(n: word): string;
    {
      ptrstr   converts a pointer to a string, format xxxx:xxxx
      ival     reads i from s and returns true, if successful
      hval     reads i from hex s and returns true, if successful
      oval     reads i from hex s and returns true, if successful
      cnumval  reads i from C-style s and returns true, if successful;
               C-style strings are 0x.. for hex, 0.. for oct and .. for
               decimal numbers
      roman    converts an integer (1..3999) to a roman numeral, e.g. MCMXCII
    }

implementation

  function bcdbin(b: byte): byte;
  begin
    bcdbin := b shr 4 * 10 + b and $0f
  end;

  function binbcd(b: byte): byte;
  begin
    binbcd := b div 10 * 16 + b mod 10
  end;

  function binb(b: byte): string;
  var
    i, j: byte;
    s: string[8];
  begin
    s := '00000000';
    i := 128;
    for j := 1 to 8 do
      begin
        if(b and i) <> 0 then s[j] := '1';
        i := i shr 1;
      end;
    binb := s;
  end;

  function binw(w: word): string;
  begin
    binw := binb(hi(w)) + binb(lo(w));
  end;

  function binl(l: longint): string;
  var
    w: array[0..1] of word absolute l;
  begin
    binl := binw(w[1]) + binw(w[0]);
  end;

  function commastr(l: longint): string;
  var
    s: string;
    len,alen,num,cnt: byte;
  begin
    str(l,s);
    len := length(s);
    if (l>=0) then alen := pred(len) else alen := pred(len)-1;
    num := alen div 3;
    for cnt := 1 to num do
      insert(',',s,len-cnt*3+1);
    commastr := s
  end;

  function commarstr(r: real; d: byte): string;
  var
    s,st: string;
    len,alen,num,cnt: byte;
    rh,rt: real;
  begin
    rh := int(r);
    rt := frac(r);
    str(rh:0:0,s);
    str(rt:0:d,st);
    delete(st,1,1);
    len := length(s);
    if r >= 0 then alen := pred(len) else alen := pred(len) - 1;
    num := alen div 3;
    for cnt := 1 to num do
      insert(',',s,len - cnt * 3 + 1);
    commarstr := s + st;
  end;

  function hexit(h: byte): char;
  begin
    if h <= 9 then
      hexit := char(h + byte('0'))
    else
      if h <= 15 then
        hexit := char(h + byte('A') - 10)
      else
        hexit := '?';
  end;

  function hexb(b: byte): string;
  var
    m,n: byte;
  begin
    m := b shr 4;
    n := b and $0f;
    hexb := hexit(m) + hexit(n);
  end;

  function hexw(w: word): string;
  begin
    hexw := hexb(hi(w)) + hexb(lo(w));
  end;

  function hexl(l: longint): string;
  var
    w: array[0..1] of word absolute l;
  begin
    hexl := hexw(w[1]) + hexw(w[0]);
  end;

  function lnumstr(n: longint; w: byte): string;
  var
    s: string;
  begin
    str(n:w,s);
    lnumstr := s
  end;

  function lnumrstr(n: real; w,d: byte): string;
  var
    s: string;
  begin
    str(n:w:d,s);
    lnumrstr := s
  end;

  function numstr(n: longint): string;
  var
    s: string;
  begin
    str(n,s);
    numstr := s
  end;

  function numrstr(n: real): string;
  var
    s: string;
    l: byte;
  begin
    str(n:20:6,s);
    while (s[1]=' ') do delete(s,1,1);
    l := length(s);
    while (s[l]='0') do begin delete(s,l,1); dec(l) end;
    if (s[l]='.') then delete(s,l,1);
    if (s='-0') then s := '0';
    numrstr := s
  end;

  function octb(b: byte): string;
  begin
    octb := char(byte('0') +((b and $c0) shr 6)) +
            char(byte('0') +((b and $38) shr 3)) +
            char(byte('0') + (b and $07));
  end;

  function octw(w: word): string;
  begin
    octw := char(byte('0') +((w and $8000) shr 15)) +
            char(byte('0') +((w and $7000) shr 12)) +
            char(byte('0') +((w and $0e00) shr  9)) +
            char(byte('0') +((w and $01c0) shr  6)) +
            char(byte('0') +((w and $0038) shr  3)) +
            char(byte('0') + (w and $0007));
  end;

  function octl(l: longint): string;
  begin
    octl := char(byte('0') +((l and $c0000000) shr 15)) +
            char(byte('0') +((l and $38000000) shr 12)) +
            char(byte('0') +((l and $07000000) shr  9)) +
            char(byte('0') +((l and $00e00000) shr  6)) +
            char(byte('0') +((l and $001c0000) shr  3)) +
            char(byte('0') +((l and $00038000) shr 15)) +
            char(byte('0') +((l and $00007000) shr 12)) +
            char(byte('0') +((l and $00000e00) shr  9)) +
            char(byte('0') +((l and $000001c0) shr  6)) +
            char(byte('0') +((l and $00000038) shr  3)) +
            char(byte('0') + (l and $00000007));
  end;

  function ptrstr(p: pointer): string;
  begin
    ptrstr := hexw(seg(p^)) + ':' + hexw(ofs(p^));
  end;

  {local} function conststr(c: char; n: byte): string;
  var
    s: string;
  begin
    s[0] := chr(n);
    fillchar(s[1],n,c);
    conststr := s
  end;

  function roman(n: word): string;
    function romandigit(one,five,ten: char; n: byte): string;
    begin
      case n of
        0   : romandigit := '';
        1..3: romandigit := conststr(one, n);
        4   : romandigit := one + five;
        5..8: romandigit := five + conststr(one, n - 5);
        9   : romandigit := one + ten;
      end
    end;
  var
    s: string;
  begin
    if n > 3999 then begin roman := ''; exit; end;
    s := romandigit('M',' ',' ',n div 1000);
    n := n mod 1000;
    s := s + romandigit('C','D','M',n div 100);
    n := n mod 100;
    s := s + romandigit('X','L','C',n div 10);
    n := n mod 10;
    s := s + romandigit('I','V','X',n);
    roman := s
  end;

  function ival(s: string; var i: longint): boolean;
  var
    code: integer;
  begin
    val(s,i,code);
    ival := (code=0)
  end;

  function hval(s: string; var i: longint): boolean;
  var
    h: longint;
    function hdig(c: char): longint;
    begin
      Case UpCase(c) Of
        '0'..'9' : hdig := Ord(c) - Ord('0');
        'A'..'Z' : hdig := Ord(UpCase(c)) - Ord('A') + 10;
        Else hdig := -1;
      End;
    end;
  begin
    i := 0;
    while (length(s)>0) do begin
      h := hdig(s[1]);
      if (h<0) then begin hval := false; exit end;
      i := 16*i+h;
      delete(s,1,1)
    end;
    hval := true
  end;

  function oval(s: string; var i: longint): boolean;
  var
    h: longint;
    function odig(c: char): longint;
    begin
      Case c Of
        '0'..'7' : odig := Ord(c) - Ord('0');
        Else odig := -1;
      End
    end;
  begin
    i := 0;
    while (length(s)>0) do begin
      h := odig(s[1]);
      if (h<0) then begin oval := false; exit end;
      i := 8*i+h;
      delete(s,1,1)
    end;
    oval := true
  end;

  function cnumval(s: string; var i: longint): boolean;
  begin
    if (s[1]='0') then begin
      if (s[2]='x') then cnumval := hval(copy(s,3,255),i)
                    else cnumval := oval(s,i)
    end else cnumval := ival(s,i)
  end;

  function zerostr(l: longint; w: byte): string;
  var
    s  : string;
    cnt: byte;
  begin
    str(l:w,s);
    for cnt := 1 to length(s) do
      if s[cnt] = ' ' then s[cnt] := '0';
    zerostr := s;
  end;

  function zerorstr(r: real; w,d: byte): string;
  var
    s  : string;
    cnt: byte;
  begin
    str(r:w:d,s);
    for cnt := 1 to length(s) do
      if s[cnt] = ' ' then s[cnt] := '0';
    zerorstr := s;
  end;

end.
