{$I SWITCHES.INC}
{ $Id: options.pas 1.3 1993/04/05 19:59:37 Ricki Exp $ }
unit options;
{ something like unix's getopt }

{
  This code is in the public domain. You may use it freely.
  There are no guarantees, either expressed or implied,
  as to either merchantability or fitness for a particular
  purpose. The author's liability is limited to the amount
  you paid for it.

  Richard Breuer

  Brunssumstrasse 6
  5100 Aachen
  Germany

  Phone:  +49/241/85605
  Fax:    +49/241/8021329
  Email:  ricki@pool.informatik.rwth-aachen.de
}

{
        $Log: options.pas $
        Revision 1.3  1993/04/05  19:59:37  Ricki
        address change

        Revision 1.2  1993/03/19  17:35:39  Ricki
        new lazy mode
        parse only once

        Revision 1.1  1993/01/22  00:54:41  Ricki
        Initial revision
}

interface

  uses
    cmscan;

  const
    pversion = '$Id: options.pas 1.3 1993/04/05 19:59:37 Ricki Exp $';

  type
    namestring = string[127]; { as cmdstring }

  var
    optarg: namestring;
    lazy,parsed: boolean; { parsed is true after parse has been called;
    			    if lazy is true, illegal options will not lead
			      to error messages }

  procedure parse(optstr: string; var nopt,nnam: byte);
  function getopt(n: byte): char;
  function getname(n: byte): namestring;
  procedure setlazy(b: boolean);
    {
      parse   parse the command line for options and filenames. optstr
              contains characters being the options, optionally followed
              by : to determine that there may be a string parameter.
              illegal options --> err. If parse has once been called
              with success, it only returns nopt and nnam for any further
              call
      getopt  get the n-th option char. optarg will be set appropriately
              (empty, if no argument is specified). if n is 0, or too big,
              #0 is returned
      getname get the n-th name argument, #0 if there is none
      setlazy sets the global lazy value
    }

implementation

  uses
    standard;

  const
    max_names = 20;
    max_opts  = 20;
    max_args  = 10;

  var
    namf: array [1..max_names] of cmdstring;
    optf: array [1..max_opts] of record
      c: char;
      a: byte; { index in argf (0 if none= }
    end;
    argf: array [1..max_args] of cmdstring;

    n_nam,n_opt,n_arg: integer;

  procedure parse(optstr: string; var nopt,nnam: byte);
  var
    p,optlen,i,j,len: byte;
    s: cmdstring;
  begin
    { do the parsing only once! }
    if (not parsed) then begin
      n_opt := 0; n_nam := 0; n_arg := 0;
      optlen := length(optstr);
      { loop over the command line parameters }
      for i:=1 to cmcount do begin
        { get the parameter }
        s := cmstr(i);
        if (s='') or (s='-') then begin
          { the empty string }
          inc(n_nam); if (n_nam>max_names) then err('Too many names');
          namf[n_nam] := ''
        end else if (s[1]='-') or (s[1]='/') then begin
          { options starting with - }
          { loop over the characters in s }
          j := 2; len := length(s);
          while (j<=len) do begin
            { check if its a valid option char }
            p := pos(s[j],optstr);
            if (p=0) and (not lazy) then err('Illegal option -'+s[j])
            else begin
              { else store it }
              inc(n_opt); if (n_opt>max_opts) then err('Too many options');
              optf[n_opt].c := s[j];
              { does the option have parameters ? }
              if (p<optlen) and (optstr[p+1]=':') then begin
                { store the argument. there cannot be anymore options in this
                  string }
                inc(n_arg); if (n_arg>max_args) then err('Too many arguments');
                argf[n_arg] := copy(s,j+1,255);
                optf[n_opt].a := n_arg;
                j := len
              end else optf[n_opt].a := 0
            end;
            inc(j)
          end
        end else begin
          { a string }
          inc(n_nam); if (n_nam>max_names) then err('Too many names');
          namf[n_nam] := s
        end
      end
    end;

    { set the appropriate output values }
    nopt := n_opt;
    nnam := n_nam
  end;

  function getopt(n: byte): char;
  begin
    if (n=0) or (n>n_opt) then getopt := #0
    else begin
      getopt := optf[n].c;
      if (optf[n].a>0) then optarg := argf[optf[n].a]
                       else optarg := ''
    end
  end;

  function getname(n: byte): namestring;
  begin
    if (n=0) or (n>n_nam) then getname := #0
                          else getname := namf[n]
  end;

  procedure setlazy(b: boolean);
  begin lazy := b end;

begin
  parsed := false;
  lazy := false
end.
