head	1.4;
access;
symbols
	rutils40:1.2;
locks;
comment	@        @;


1.4
date	93.04.05.19.56.54;	author Ricki;	state Exp;
branches;
next	1.3;

1.3
date	93.03.27.15.39.12;	author Ricki;	state Exp;
branches;
next	1.2;

1.2
date	93.03.19.17.39.29;	author Ricki;	state Rel;
branches;
next	1.1;

1.1
date	93.01.22.00.54.44;	author Ricki;	state Exp;
branches;
next	;


desc
@procedures for processing the command line
@

1.4
log
@recursion also, if m_dir is set and a directory is specified
@
text
@{$I SWITCHES.INC}
{ $Id: cmdline.pas 1.3 1993/03/27 15:39:12 Ricki Exp $ }
unit cmdline;
{ procedures for processing the command line }

{
  This code is in the public domain. You may use it freely.
  There are no guarantees, either expressed or implied,
  as to either merchantability or fitness for a particular
  purpose. The author's liability is limited to the amount
  you paid for it.

  Richard Breuer

  Brunssumstrasse 6
  5100 Aachen
  Germany

  Phone:  +49/241/85605
  Fax:    +49/241/8021329
  Email:  ricki@@pool.informatik.rwth-aachen.de
}

{
        $Log: cmdline.pas $
        Revision 1.3  1993/03/27  15:39:12  Ricki
        fixed the '- -' bug

        Revision 1.2  1993/03/19  17:39:29  Ricki
        bugfix for listfile handling

        Revision 1.1  1993/01/22  00:54:44  Ricki
        Initial revision
}

interface

  uses
    numbers,
    options,
    files,
    dos;

  const
    pversion = '$Id: cmdline.pas 1.3 1993/03/27 15:39:12 Ricki Exp $';

    m_none     = $0000;
    m_stdin    = $0001;
    m_pipe     = $0002;
    m_quiet    = $0004;
    m_recurse  = $0008;
    m_dir      = $0010;
    m_sysfile  = $0020;
    m_readonly = $0040;

  type
    namestring = string [127]; { as options.namestring }

    TPreProc = procedure(val: byte);
    TPostProc = procedure;
    TopProc = procedure(opt: char; optarg: namestring);
    TfnProc = procedure(var s: searchrec; p: pathstr); { as files.TfileProc }

  procedure process(optstring: string;
                    pre_op: TPreProc; proc_op: TopProc; post_op: TPostProc;
                    pre_fn: TPreproc; proc_fn: TfnProc; post_fn: TPostProc;
                    mode: word);
  {
    process - calls pre_op(nopt) (for option handling initialization)
            - calls proc_op for each option in the command line together
              with the appropriate optarg ('', if none)
            - calls proc_op(#0,'') if there are neither options nor names
              and redirection is not active
            - calls post_op (for additional option handling)
            - calls pre_np(nnam) (for file handling initialization)
            - calls proc_np for each expanded name in the command line;
              expansion means:
                if the filename starts with @@, a list file is opened & read
                if there are wildcard chars * or ?, a loop is started
                one single file is opened & processed
            - calls post_op (for addition file handling)

            mode is a mask specifying the behaviour of process
                m_stdin   pass '' as name to np; if not set, '' raises an
                          error
                m_pipe    pass '' as name to np, if the input is redirected;
                          if not set, redirection raises an error
                m_quiet   do not warn, if one of the filenames does not
                          exist; if not set, a warning is raised if a file
                          does not exist
                m_recurse if set, each pathname is interpreted as
                          path/file.ext. Then np is called for each file.ext
                          under path recursively
                m_dir     if set, directory names are passed to np, otherwise
                          they are expanded to dir\*.*
                m_sysfile if set, hidden and system files are processed
  }

implementation

  uses
    outcon,
    dostools,
    strings,
    standard;

  var
    first_stdin: boolean;

  {local} procedure assign_stdin(var sr: searchrec);
  { fills sr with the appropriate values for stdin. if stdin is used for
    the first time, standard input is read, otherwise the console is read.
    this means that PROG - - will result in reading from a pipe, and after
    that reading from the keyboard }
  begin
    if (first_stdin) then sr.name := ''
                     else sr.name := 'CON';
    first_stdin := false;
    sr.attr := archive;
    sr.size := 0;
    sr.time := 0;
  end;

  {local} procedure assign_dir(var sr: searchrec; n: namestring);
  { fills sr with the appropriate values for a directory }
  begin
    sr.name := n;
    sr.attr := directory;
    sr.size := 0;
    sr.time := 0;
  end;

  procedure process(optstring: string;
                    pre_op: TPreProc; proc_op: TopProc; post_op: TPostProc;
                    pre_fn: TPreproc; proc_fn: TfnProc; post_fn: TPostProc;
                    mode: word);
  var
    nopt,nnam: byte;
    i,code: byte;
    fn: namestring;
    redirected: boolean;
    sr: searchrec;
    mask: word;
    fdir: dirstr;
    fnam: namestr;
    fext: extstr;
    codeline: longint;
  begin
    { parse the command line }
    parse(optstring,nopt,nnam);

    { determine whether the input has been redirected }
    redirected := InpRedirected;

    { setup the appropriate file mask }
    mask := archive;
    if (mode and m_sysfile=m_sysfile) then mask := mask or hidden or sysfile;
    if (mode and m_readonly=m_readonly) then mask := mask or readonly;
    if (mode and m_dir=m_dir) then mask := mask or directory;

    { process the options }
    pre_op(nopt);
    if (nopt+nnam=0) and (not redirected) then proc_op(#0,'')
    else for i:=1 to nopt do proc_op(getopt(i),optarg);
    post_op;

    pre_fn(nnam);

    { special case: no names, input redirected }
    if (nnam=0) and (redirected) then begin
      if (mode and m_pipe=m_pipe) then begin
        assign_stdin(sr);
        proc_fn(sr,'')
      end else begin
        if (mode and m_quiet=m_none) then warn('Piping is not allowed')
      end

    { no names? no pipe? no job! }
    end else if (nnam=0) then begin
      if (mode and m_quiet=0) then warn('Nothing to do')
    end

    { loop over the names }
    else for i:=1 to nnam do begin
      fn := getname(i);

      { handle directory names }
      if (isdir(fn)) then begin
        if (mode and m_dir=0) then begin
          { a directory name means dir\*.* }
          if (mode and m_recurse=m_recurse) then begin
            forallrec(fn,'*.*',mask,proc_fn,code)
          end else begin
            if (fn[length(fn)]='\') then fn := fn+'*.*'
                                    else fn := fn+'\*.*';
            forall(fn,mask,proc_fn,code)
          end
        end else begin { mode and m_dir<>0 }
          assign_dir(sr,fn);
          proc_fn(sr,'');
          { recurse here, too }
          if (mode and m_recurse=m_recurse) then begin
            forallrec(fn,'*.*',mask or directory,proc_fn,code)
          end else code := 0
        end;
        if (code<>0) then begin
          if (mode and m_quiet=0) then begin
            err('Directory loop failed for '+fn+': '+fileerrstr(code))
          end
        end
      end else begin { not isdir(fn) }
        { stdin? }
        if (fn='') then begin
          if (mode and m_stdin=m_stdin) then begin
            assign_stdin(sr);
            proc_fn(sr,'');
            write('')
          end
          else begin
            if (mode and m_quiet=0) then begin
              err('Standard input is not allowed')
            end
          end
        end
        { handle listfile names }
        else if (fn[1]='@@') then begin
	  forlistfile(copy(fn,2,255),mask,proc_fn,code,codeline);
          if (code<>0) then begin
            if (mode and m_quiet=0) then begin
              err('Listfile loop failed for '+fn+': '+fileerrstr(code)+
            ' (detected in line '+numstr(codeline)+')')
            end
          end
        end
        else begin { not stdin }
          { try to start a loop over fn }
          if (mode and m_recurse=0) then begin
            forall(fn,mask,proc_fn,code)
          end
          else begin
            { split name+extension from fn, because they are the mask;
              the directory component is the path }
            fsplit(fn,fdir,fnam,fext);
            forallrec(fdir,fnam+fext,mask,proc_fn,code)
          end;
          if (code<>0) then begin
            if (mode and m_quiet=0) then begin
              err('Filename loop failed for '+fn+': '+fileerrstr(code))
            end
          end
        end
      end
    end;

    post_fn
  end;

begin
  first_stdin := true
end.
@


1.3
log
@fixed the '- -' bug
@
text
@d2 1
a2 1
{ $Id: cmdline.pas 1.2 1993/03/19 17:39:29 Ricki Exp $ }
d21 1
a21 1
  Email:  ricki@@rwthi3.informatik.rwth-aachen.de
d26 3
d45 1
a45 1
    pversion = '$Id: cmdline.pas 1.2 1993/03/19 17:39:29 Ricki Exp $';
d193 1
a193 2
          end
          else begin
a196 5
          end;
          if (code<>0) then begin
            if (mode and m_quiet=0) then begin
              err('Directory loop failed for '+fn+': '+fileerrstr(code))
            end
d198 1
a198 2
        end
        else begin { mode and m_dir<>0 }
d200 10
a209 1
          proc_fn(sr,'')
@


1.2
log
@bugfix for listfile handling
@
text
@d2 1
a2 1
{ $Id: CMDLINE.PAS 1.1 1993/01/22 00:54:44 Ricki Exp $ }
d25 4
a28 1
        $Log: CMDLINE.PAS $
d42 1
a42 1
    pversion = '$Id: CMDLINE.PAS 1.1 1993/01/22 00:54:44 Ricki Exp $';
d104 3
d108 4
a111 1
  { fills sr with the appropriate values for stdin }
d113 3
a115 1
    sr.name := '';
d211 2
a212 1
            proc_fn(sr,'')
d253 2
a255 1

@


1.1
log
@Initial revision
@
text
@d1 2
a2 1
{ $Id$ }
d25 3
a27 1
        $Log$
d33 1
d39 1
a39 1
    pversion = '$Id$';
d133 1
d145 1
d195 1
a195 2
      end
      else begin { not isdir(fn) }
d205 10
@

