head	1.3;
access;
symbols
	rutils40:1.2;
locks;
comment	@        @;


1.3
date	93.04.05.19.59.18;	author Ricki;	state Exp;
branches;
next	1.2;

1.2
date	93.03.19.17.33.19;	author Ricki;	state Rel;
branches;
next	1.1;

1.1
date	93.01.22.00.54.17;	author Ricki;	state Exp;
branches;
next	;


desc
@various calculations concerning dates in the range 1-2999;
all calculations are based on the fact, that the gregorian
reformation removed the 11 days from Sep 3 to Sep 13 1752 }
@

1.3
log
@address change
@
text
@{$I SWITCHES.INC}
{ $Id: dates.pas 1.2 1993/03/19 17:33:19 Ricki Exp $ }
unit Dates;
{ various calculations concerning dates in the range 1-32767;
  all calculations are based on the fact, that the gregorian
  reformation removed the 11 days from Sep 3 to Sep 13 1752 }

{
  This code is in the public domain. You may use it freely.
  There are no guarantees, either expressed or implied,
  as to either merchantability or fitness for a particular
  purpose. The author's liability is limited to the amount
  you paid for it.

  Richard Breuer

  Brunssumstrasse 6
  5100 Aachen
  Germany

  Phone:  +49/241/85605
  Fax:    +49/241/8021329
  Email:  ricki@@pool.informatik.rwth-aachen.de


  Most of the code was derived from Usenet articles cited in
  the implementation section. Note, that all procedures expect a
  valid date as input, ie. a date, for which validdate(d,m,y)=true.
}

{
        $Log: dates.pas $
        Revision 1.2  1993/03/19  17:33:19  Ricki
        *** empty log message ***

        Revision 1.1  1993/01/22  00:54:17  Ricki
        Initial revision
}

interface

  const
    pversion = '$Id: dates.pas 1.2 1993/03/19 17:33:19 Ricki Exp $';

  type
    daystring = string[10];
    monthstring = string[9];
    datestring = string[10];

  function leapyear(year: integer): boolean;
  function dayinyear(day,month,year: integer): integer;
  function dayinweek(day,month,year: integer): integer;
  function weekinyear(day,month,year: integer): integer;
  function daysinmonth(month,year: integer): integer;
    {
      leap_year    accounts for the gregorian reformation in 1752
      dayinyear    returns the 1 based day number within the year, sometimes
                   referred to as the julian date
      dayinweek    returns a zero based day number for any date from 1 jan 1
                   to 31 dec 9999. Assumes the gregorian reformation
                   eliminates 3 sep 1752 through 13 sep 1752. Returns
                   THURSDAY (4) for all missing days
                     Sunday    0
                     Monday    1
                     Tuesday   2
                     Wednesday 3
                     Thursday  4
                     Friday    5
                     Saturday  6
      weekinyear   returns the number of the week according to DIN 1355
      daysinmonth  returns the number of days in month m, accounts for
                   leapyears and the gregorian reformation
    }

  procedure Easter(year: integer; var day,month: integer);
  procedure GoodFriday(year: integer; var day,month: integer);
  procedure HolySaturday(year: integer; var day,month: integer);
  procedure EasterMonday(year: integer; var day,month: integer);
  procedure WhitSunday(year: integer; var day,month: integer);
  procedure WhitMonday(year: integer; var day,month: integer);
  procedure NewYear(year: integer; var day,month: integer);
  procedure NewYearsEve(year: integer; var day,month: integer);
  procedure AshWednesday(year: integer; var day,month: integer);
  procedure PalmSunday(year: integer; var day,month: integer);
  procedure WhiteSunday(year: integer; var day,month: integer);
  procedure ChristsAssumption(year: integer; var day,month: integer);
  procedure CorpusChristi(year: integer; var day,month: integer);
  procedure MariaeAssumption(year: integer; var day,month: integer);
  procedure Advent1(year: integer; var day,month: integer);
  procedure Advent2(year: integer; var day,month: integer);
  procedure Advent3(year: integer; var day,month: integer);
  procedure Advent4(year: integer; var day,month: integer);
  procedure Christmas(year: integer; var day,month: integer);
  procedure Christmas2(year: integer; var day,month: integer);
  procedure RepentanceDay(year: integer; var day,month: integer);
  procedure ThreeKings{?}(year: integer; var day,month: integer);
  procedure AllSaintsDay(year: integer; var day,month: integer);
    { ... }

  function validdate(day,month,year: integer): boolean;
  procedure adddays(var day,month,year: integer; add: longint);
  procedure dayinyear2dmy(year,diy: integer; var day,month: integer);
  function daydiff(d1,m1,y1,d2,m2,y2: integer): longint;
    {
      validdate     returns whether a date is ok; accounts for the
                    reformation and leap years; valid dates may be in
                    the range 1/1/1..9/2/1752,9/14/1752..12/31/2999
      adddays       computes day,month,year for an addition of add days;
                    note: add may be negative; note: adddays does also
                    work when the reformation date Sep 1752 is stepped over
      dayinyear2dmy computes day,month from the given day_in_year diy
      daydiff       computes the difference d1.m1.y1-d2.m2.y2 in days;
                    note: the difference may (of course) be negative;
                    note: daydiff does also work if the reformation date
                    Sep 1752 is stepped over
    }

  function daystr(d: integer): daystring;
  function monthstr(m: integer): monthstring;
  function gdaystr(d: integer): daystring;
  function gmonthstr(m: integer): monthstring;
  function date2str(day,month,year: integer): datestring;
  function gdate2str(day,month,year: integer): datestring;
  function edate2str(day,month,year: integer): datestring;
  function sdate2str(day,month,year: integer): datestring;
  function curdate: datestring;
  function gcurdate: datestring;
  function ecurdate: datestring;
  function scurdate: datestring;
    {
      daystr        returns (English) name of a day of the week,
                    Sunday=0 as above
      monthstr      returns (English) name of month
      gdaystr       German daystr
      gmonthstr     German monthstr
      @@daystr       *** if you want to see your language here, give
      @@monthstr     *** me a procedure (or at least the names :-))

      date2str      returns a string mm/dd/yyyy (American)
      gdate2str     returns a string dd.mm.yyyy (German)
      edate2str     returns a string dd/mm/yyyy (English)
      sdate2str     returns a string yyyy/mm/dd (for sorting)
      @@date2str     *** same as above

      curdate       returns a string mm/dd/yyyy (American)
      gcurdate      returns a string dd.mm.yyyy (German)
      ecurdate      returns a string dd/mm/yyyy (English)
      scurdate      returns a string yyyy/mm/dd (for sorting)
      @@curdate      *** same as above
    }

implementation

  uses
    standard,
    numbers,
    dos;

  const
    minyear = 1;
    maxyear = 32767;

  { The following comments are taken from a usenet article, the
    procedures are a straight port of the C functions in it.

    From: kim@@Software.Mitel.COM (Kim Letkeman)
    Newsgroups: comp.lang.pascal
    Subject: Re: Day of week on a given date
    Message-ID: <KIM.92Nov17115401@@kim.Software.Mitel.COM>
    Date: 17 Nov 92 16:54:01 GMT
    References: <1992Nov17.042319.29002@@cs.unca.edu>
    Sender: kim@@Software.Mitel.COM
    Organization: MITEL Public Switching, Kanata, Ontario, Canada
    In-reply-to: snodgras@@cs.unca.edu's message of 17 Nov 92 04:23:19 GMT

    [...]
    I take 1 jan 1 to be a Saturday because that's what cal says and I
    couldn't change that even if I was dumb enough to try. From this we
    can easily calculate the day of week for any date. The algorithm for a
    zero based day of week:

	    calculate the number of days in all prior years (year-1)*365
	    add the number of leap years (days?) since year 1
		    (not including this year as that is covered later)
	    add the day number within the year
		    this compensates for the non-inclusive leap year
		    calculation
	    if the day in question occurs before the gregorian reformation
		    (3 sep 1752 for our purposes), then simply return
		    (value so far - 1 + SATURDAY's value of 6) modulo 7.
	    if the day in question occurs during the reformation (3 sep 1752
		    to 13 sep 1752 inclusive) return THURSDAY. This is my
		    idea of what happened then. It does not matter much as
		    this program never tries to find day of week for any day
		    that is not the first of a month.
	    otherwise, after the reformation, use the same formula as the
		    days before with the additional step of subtracting the
		    number of days (11) that were adjusted out of the calendar
		    just before taking the modulo.

    It must be noted that the number of leap years calculation is
    sensitive to the date for which the leap year is being calculated. A
    year that occurs before the reformation is determined to be a leap
    year if its modulo of 4 equals zero. But after the reformation, a year
    is only a leap year if its modulo of 4 equals zero and its modulo of
    100 does not. Of course, there is an exception for these century
    years. If the modulo of 400 equals zero, then the year is a leap year
    anyway. This is, in fact, what the gregorian reformation was all about
    (a bit of error in the old algorithm that caused the calendar to be
    inaccurate.)
    [...] }

  const
    JANUARY = 1;
    THURSDAY = 4;                        { for reformation }
    SATURDAY = 6;                        { 1 Jan 1 was a saturday }
    {
      the following number in the article seems to be wrong!
      I hopefully corrected it to the right value
      FIRST_MISSING_DAY: longint = 639787;
    }
    FIRST_MISSING_DAY: longint = 639799; { 3 Sep 1752 }
    NUMBER_MISSING_DAYS = 11;            { 11 day correction }
    MONTH_REFORMATION = 9;               { Sep }
    YEAR_REFORMATION = 1752;             { 1752 }

  const
    days_in_month: array [false..true,0..12] of byte =
      ((0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31),
       (0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31));
    days_added: array [false..true,0..12] of integer =
      ((0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365),
       (0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366));

  function leapyear(year: integer): boolean;
  begin
    if (year<=YEAR_REFORMATION) then
      leapyear := (year mod 4=0)
    else
      leapyear := ( (year mod 4=0) and (year mod 100<>0) )
                 or (year mod 400=0)
  end;

  function dayinyear(day,month,year: integer): integer;
  begin dayinyear := day+days_added[leapyear(year),month-1];
  end;

  {local} function centuries_since_1700(year: integer): integer;
  { centuries_since_1700 returns the number of xx00 years that have occured
    since 1700 *not inclusive* }
  begin
    if (year>1700) then centuries_since_1700 := ((year div 100)-17)
                   else centuries_since_1700 := 0
  end;

  {local} function quad_centuries_since_1700(year: integer): integer;
  { quad_centuries_since_1700 returns the number of xx00 years whose modulo
    of 400 == 0, also since 1700 }
  begin
    if (year>1600) then quad_centuries_since_1700 := ((year-1600) div 400)
                   else quad_centuries_since_1700 := 0
  end;

  {local} function leap_years_since_year_1(year: integer): integer;
  { leap_years_since_year_1 returns the number of leap years between year 1
    and this year *not inclusive* }
  begin
    dec(year);
    leap_years_since_year_1 := (year div 4)-centuries_since_1700(year)
	                       +quad_centuries_since_1700(year)
  end;

  {local} function days_since_year_1(day,month,year: integer): longint;
  { the number of days since year 1, *not* accounting for the
    gregorian reformation }
  var
    temp: longint;
  begin
    temp := year-1;
    temp := temp*365+leap_years_since_year_1(year)
                    +dayinyear(day,month,year);
    days_since_year_1 := temp
  end;

  {local} procedure rev_days_since_year_1(t: longint;
                                          var day,month,year: integer);
  { the reverse function to the above }
  var
    approx: longint;
    leap: boolean;
  begin
    if (t<=0) then err('Illegal date specified');
    { year is a year that is greater or equal to the desired one }
    year := 1+(t div 365);
    { decrease year, until 1.1.year is earlier than the date given by t }
    repeat
      approx := days_since_year_1(1,1,year);
      if (approx>t) then dec(year)
    until (approx<=t);
    leap := leapyear(year);
    { the reduced t contains the remaining number of days within year }
    dec(t,approx);
    { for the rest see also: dayinyear2dmy }
    month := 12;
    while (t<days_added[leap,month]) do dec(month);
    day := 1+t-days_added[leap,month];
    inc(month)
  end;

  function dayinweek(day,month,year: integer): integer;
  var
    temp: longint;
  begin
    temp := days_since_year_1(day,month,year);
    if (temp<FIRST_MISSING_DAY) then
      dayinweek := (temp-1+SATURDAY) mod 7
    else if (temp>=(FIRST_MISSING_DAY+NUMBER_MISSING_DAYS)) then
      dayinweek := ((temp-1+SATURDAY)-NUMBER_MISSING_DAYS) mod 7
    else
      dayinweek := THURSDAY
  end;

  { The following comments and procedures are taken from another usenet
    article. I removed the calculations for Tag der deutschen Einheit
    and Tag der Arbeit, because they were not avaliable throughout the
    centuries.

    Date: Thu, 08 Oct 1992 22:21:00 CET
    From: p.mandrella@@sps-hh.zer.de
    Reply-To: p.mandrella@@sps-hh.zer.de
    Distribution: zer
    Message-ID: <4nTz_oVcU~@@SPS-HH.zer.sub.org>
    Newsgroups: zer.z-netz.rechner.ibm.programmieren
    Subject: Re: Kalenderarithmetik
    Organization: Stack's Point Server, Hamburg,FRG
    Content-Length: 11451
    X-Gateway: UU uzercp [UZERCP V3.99]


    ======================================
    Datumsarithmetik mit den PC und Pascal
    ======================================

    Dieser Informationstext soll allen Programmierern helfen, die mit
    der Berechnung von Tagesdaten oder der Logik der kirchlichen Feier-
    tage kaempfen. Diese Text kann frei verteilt werden, solange die
    folgenden Informationen nicht daraus entfernt werden.

    Copyright (c) 1992 fuer diese Zusammenstellung und den erlaeuternden
    Text sowie evtl. erfolgte Korrekturen by Armin Hanisch (2:246/41)
    Ich moechte allen danken, die durch ihre e-Mails und sonstige Bei-
    traege, insbesondere der Veroeffentlichung von Sources diese Zu-
    sammenstellung erst moeglich gemacht haben. Nach Informationsstand
    des Autors haben alle Autoren diese Routinen als Public Domain frei-
    gegeben. Ich uebernehme allerdings weder dafuer noch fuer die korekte
    Berechnung irgendwelcher Daten eine Garantie. Ich habe allerdings
    einige Stunden an Testarbeit investiert, die Daten stimmen!

    Sources und Alogrithmen von folgenden Personen wurden ausgewertet,
    zur Verfuegung gestellt und teilweise korrigert oder an einen fuer
    diesen Text einheitlichen Stil bzw. Datentyp angepasst:

    Armin Hanisch - Feiertagsberechnungen
    Bernd Strehuber - Feiertagsberechnungen
    Carley Phillips - Jul. Berechnungen
    Jeff Duntemann - Wochentagsberechnung
    Judson McClendon - Osterberechnung
    Martin Austermeier - Tagesberechnungen
    Paul Schlyter - Osterberechnung
    Pit Biernath - Jul. Berechnungen
    Scott Bussinger - Jul. Berechnungen
    [...] }

  { Berechnung der Kalenderwoche:
    Die Woche 1 ist die Woche, die den ersten Donnerstag des Jahres
    enthaelt, also mehr als die Haelfte diesem Jahr angehoert.
    Ist der 01.01. ein Mo-Mi, dann liegt der 01.01. in der letzten
    Woche des vergangenen Jahres. (DIN 1355) }

  function weekinyear(day,month,year: integer): integer;
  const
    table1: array [0..6] of shortint = ( -1,  0,  1,  2,  3, -3, -2);
    table2: array [0..6] of shortint = ( -4,  2,  1,  0, -1, -2, -3);
  var
    doy1,doy2: integer;
  begin
    doy1 := dayinyear(day,month,year)+table1[dayinweek(1,1,year)];
    doy2 := dayinyear(day,month,year)+table2[dayinweek(day,month,year)];
    if (doy1<=0) then weekinyear := weekinyear(31,12,year-1)
    else if (doy2>=dayinyear(31,12,year)) then weekinyear := 1
    else weekinyear := (doy1-1) div 7 + 1;
  end;

  { OSTERBERECHNUNG:
    Dieser Algorithmus basiert nicht auf der Berechnung von Gauss und
    kommt ohne Ausnahmen aus (lt. Paul Schlyter) }
  procedure Easter(year: integer; var day,month: integer);
  var
    a,b,c,d,e,f,g,h,i,k,l,m,res: integer;
  begin
     a := year mod 19;
     b := year div 100;
     c := year mod 100;
     d := b div 4;
     e := b mod 4;
     f := ( b + 8 ) div 25;
     g := ( b - f + 1 ) div 3;
     h := ( 19 * a + b - d - g + 15 ) mod 30;
     i := c div 4;
     k := c mod 4;
     l := ( 32 + 2 * e + 2 * i - h - k ) mod 7;
     m := ( a + 11 * h + 22 * l ) div 451;
     res := h + l - 7 * m + 22;
     if (res>31) then begin
       month := 4; { april }
       day := res-31
     end else begin
       month := 3; { march }
       day := res
     end
  end;

  { Aschermittwoch:      40 Tage vor dem Ostersonntag,
                         dann zurueckgehen bis zum Mittwoch }
  procedure AshWednesday(year: integer; var day,month: integer);
  var
    dow: integer;
  begin
    easter(year,day,month);
    adddays(day,month,year,-40);
    dow := dayinweek(day,month,year);
    if (dow<3) then inc(dow,7);
    adddays(day,month,year,dow-3)
  end;

  { Palmsonntag:         Der Sonntag vor dem Ostersonntag, die Berechnung
                         ist damit trivial }
  procedure PalmSunday(year: integer; var day,month: integer);
  begin
    easter(year,day,month);
    adddays(day,month,year,-7)
  end;

  { Weisser Sonntag:     Der Sonntag nach Ostern, ebenfalls simpel }
  procedure WhiteSunday(year: integer; var day,month: integer);
  begin
    easter(year,day,month);
    adddays(day,month,year,7)
  end;

  { Christi Himmelfahrt: 39 Tage nach dem Ostersonntag oder anders gesagt,
                         der zweite Donnerstag vor Pfingsten }
  procedure ChristsAssumption(year: integer; var day,month: integer);
  begin
    easter(year,day,month);
    adddays(day,month,year,39)
  end;

  { Pfingsten:           49 Tage nach dem Ostersonntag }
  procedure WhitSunday(year: integer; var day,month: integer);
  begin
    easter(year,day,month);
    adddays(day,month,year,49)
  end;

  { Fronleichnam:        60 Tage nach dem Ostersonntag }
  procedure CorpusChristi(year: integer; var day,month: integer);
  begin
    easter(year,day,month);
    adddays(day,month,year,60)
  end;

  { Maria Himmelfahrt:   Fest am 15. August (nicht ueberall Feiertag!) }
  procedure MariaeAssumption(year: integer; var day,month: integer);
  begin
    day := 15;
    month := 8
  end;

  { 1. Advent:           Vom 24.12. zurueck bis zum naechsten Sonntag,
                         dann noch drei Wochen zurueck }
  procedure Advent1(year: integer; var day,month: integer);
  var
    dow: integer;
  begin
    day := 24;
    month := 12;
    dow := dayinweek(day,month,year);
    if (dow=0) then dow := 7;
    adddays(day,month,year,-dow-21)
  end;

  procedure Advent2(year: integer; var day,month: integer);
  var
    dow: integer;
  begin
    day := 24;
    month := 12;
    dow := dayinweek(day,month,year);
    if (dow=0) then dow := 7;
    adddays(day,month,year,-dow-14)
  end;

  procedure Advent3(year: integer; var day,month: integer);
  var
    dow: integer;
  begin
    day := 24;
    month := 12;
    dow := dayinweek(day,month,year);
    if (dow=0) then dow := 7;
    adddays(day,month,year,-dow-7)
  end;

  procedure Advent4(year: integer; var day,month: integer);
  var
    dow: integer;
  begin
    day := 24;
    month := 12;
    dow := dayinweek(day,month,year);
    if (dow=0) then dow := 7;
    adddays(day,month,year,-dow)
  end;

  { Buss- und Bettag:    Der vorvorige Mittwoch vor dem 1. Advent, also
                         vom 1. Advent aus den Mittwoch suchen, dann noch
                         eine Woche zurueck }
  procedure RepentanceDay(year: integer; var day,month: integer);
  var
    dow: integer;
  begin
    Advent1(year,day,month);
    adddays(day,month,year,-4-7);
  end;

  { Hl. drei Koenige:    Fest am 06.01. }
  procedure ThreeKings(year: integer; var day,month: integer);
  begin
    day := 6;
    month := 1
  end;

  { Allerheiligen:       Fest am 01.11.}
  procedure AllSaintsDay(year: integer; var day,month: integer);
  begin
    day := 1;
    month := 11
  end;

  { ---------- The following procedures were added by me --------- }

  { Christmas: 24.12.}
  procedure Christmas(year: integer; var day,month: integer);
  begin
    day := 24;
    month := 12
  end;

  { Christmas 2: 25.12.}
  procedure Christmas2(year: integer; var day,month: integer);
  begin
    day := 25;
    month := 12
  end;

  { Karfreitag: 2 Tage (Freitag) vor Ostern }
  procedure GoodFriday(year: integer; var day,month: integer);
  begin
    Easter(year,day,month);
    adddays(day,month,year,-2)
  end;

  { Karsamstag: 1 Tag (Samstag) vor Ostern }
  procedure HolySaturday(year: integer; var day,month: integer);
  begin
    Easter(year,day,month);
    adddays(day,month,year,-1)
  end;

  { Ostermontag: 1 Tag (Montag) nach Ostern }
  procedure EasterMonday(year: integer; var day,month: integer);
  begin
    Easter(year,day,month);
    adddays(day,month,year,1)
  end;

  { Karsamstag: 1 Tag (Montag) nach Pfingsten (siehe dort) }
  procedure WhitMonday(year: integer; var day,month: integer);
  begin
    Easter(year,day,month);
    adddays(day,month,year,50)
  end;

  { Neujahr: 1.1. }
  procedure NewYear(year: integer; var day,month: integer);
  begin
    day := 1;
    month := 1
  end;

  { Silvester: 31.12. }
  procedure NewYearsEve(year: integer; var day,month: integer);
  begin
    day := 31;
    month := 12
  end;

  function validdate(day,month,year: integer): boolean;
  begin
    validdate := false;
    if (year<1) or (year>maxyear) then exit;
    if (month<1) or (month>12) then exit;
    if (day>daysinmonth(month,year)) then exit;
    if (year=YEAR_REFORMATION) and (month=MONTH_REFORMATION)
                               and (day>2) and (day<14) then exit;
    validdate := true
  end;

  function daysinmonth(month,year: integer): integer;
  begin
    if (month<1) or (month>12) then daysinmonth := 0
    else begin
      { account for Sep 1752 }
      if (year=YEAR_REFORMATION) and (month=MONTH_REFORMATION)
      then
        daysinmonth := days_in_month[leapyear(year),month]-NUMBER_MISSING_DAYS
      else
        daysinmonth := days_in_month[leapyear(year),month]
    end
  end;

  procedure adddays(var day,month,year: integer; add: longint);
  var
    temp,tempa: longint;
  begin
    temp := days_since_year_1(day,month,year);
    tempa := temp+add;

    if (temp<FIRST_MISSING_DAY) and (tempa>=FIRST_MISSING_DAY)
      then rev_days_since_year_1(tempa+NUMBER_MISSING_DAYS,day,month,year)
    else if (temp>=FIRST_MISSING_DAY+NUMBER_MISSING_DAYS)
        and (tempa<FIRST_MISSING_DAY+NUMBER_MISSING_DAYS)
      then rev_days_since_year_1(tempa-NUMBER_MISSING_DAYS,day,month,year)
    else
      rev_days_since_year_1(tempa,day,month,year)
  end;

  procedure dayinyear2dmy(year,diy: integer; var day,month: integer);
  var
    leap: boolean;
  begin
    leap := leapyear(year);
    month := 12;
    while (diy<=days_added[leap,month]) do dec(month);
    day := diy-days_added[leap,month];
    inc(month)
  end;

  function daydiff(d1,m1,y1,d2,m2,y2: integer): longint;
  var
    temp1,temp2: longint;
  begin
    temp1 := days_since_year_1(d1,m1,y1);
    temp2 := days_since_year_1(d2,m2,y2);

    if (temp1<FIRST_MISSING_DAY) and (temp2>=FIRST_MISSING_DAY)
      then daydiff := temp1-temp2+11
    else if (temp1>=FIRST_MISSING_DAY+NUMBER_MISSING_DAYS)
        and (temp2<FIRST_MISSING_DAY+NUMBER_MISSING_DAYS)
      then daydiff := temp1-temp2-11
    else
      daydiff := temp1-temp2
  end;

  function daystr(d: integer): daystring;
  begin
    case d of
      0: DayStr := 'Sunday';
      1: DayStr := 'Monday';
      2: DayStr := 'Tuesday';
      3: DayStr := 'Wednesday';
      4: DayStr := 'Thursday';
      5: DayStr := 'Friday';
      6: DayStr := 'Saturday';
    else
      DayStr := ''
    end
  end;

  function monthstr(m: integer): monthstring;
  begin
    case m of
      1 : MonthStr := 'January';
      2 : MonthStr := 'February';
      3 : MonthStr := 'March';
      4 : MonthStr := 'April';
      5 : MonthStr := 'May';
      6 : MonthStr := 'June';
      7 : MonthStr := 'July';
      8 : MonthStr := 'August';
      9 : MonthStr := 'September';
      10: MonthStr := 'October';
      11: MonthStr := 'November';
      12: MonthStr := 'December';
    else
      MonthStr:= ''
    end
  end;

  function gdaystr(d: integer): daystring;
  begin
    case d of
      0: gDayStr := 'Sonntag';
      1: gDayStr := 'Montag';
      2: gDayStr := 'Dienstag';
      3: gDayStr := 'Mittwoch';
      4: gDayStr := 'Donnerstag';
      5: gDayStr := 'Freitag';
      6: gDayStr := 'Samstag';
    else
      gDayStr := ''
    end
  end;

  function gmonthstr(m: integer): monthstring;
  begin
    case m of
      1 : gMonthStr := 'Januar';
      2 : gMonthStr := 'Februar';
      3 : gMonthStr := 'Mrz';
      4 : gMonthStr := 'April';
      5 : gMonthStr := 'Mai';
      6 : gMonthStr := 'Juni';
      7 : gMonthStr := 'Juli';
      8 : gMonthStr := 'August';
      9 : gMonthStr := 'September';
      10: gMonthStr := 'Oktober';
      11: gMonthStr := 'November';
      12: gMonthStr := 'Dezember';
    else
      gMonthStr:= ''
    end
  end;

  function date2str(day,month,year: integer): datestring;
  begin
    date2str := zerostr(month,2)+'/'+
                zerostr(day,2)  +'/'+
                zerostr(year,4)
  end;

  function gdate2str(day,month,year: integer): datestring;
  begin
    gdate2str := zerostr(day,2)  +'.'+
                 zerostr(month,2)+'.'+
                 zerostr(year,4)
  end;

  function edate2str(day,month,year: integer): datestring;
  begin
    edate2str := zerostr(day,2)  +'/'+
                 zerostr(month,2)+'/'+
                 zerostr(year,4)
  end;

  function sdate2str(day,month,year: integer): datestring;
  begin
    sdate2str := zerostr(year,4)  +'/'+
                 zerostr(month,2)+'/'+
                 zerostr(day,2)
  end;

  function curdate: datestring;
  var
    y,m,d,w: word;
  begin
    getdate(y,m,d,w);
    curdate := date2str(d,m,y)
  end;

  function gcurdate: datestring;
  var
    y,m,d,w: word;
  begin
    getdate(y,m,d,w);
    gcurdate := gdate2str(d,m,y)
  end;

  function ecurdate: datestring;
  var
    y,m,d,w: word;
  begin
    getdate(y,m,d,w);
    ecurdate := edate2str(d,m,y)
  end;

  function scurdate: datestring;
  var
    y,m,d,w: word;
  begin
    getdate(y,m,d,w);
    scurdate := sdate2str(d,m,y)
  end;


end.
@


1.2
log
@*** empty log message ***
@
text
@d2 1
a2 1
{ $Id: DATES.PAS 1.1 1993/01/22 00:54:17 Ricki Exp $ }
d23 1
a23 1
  Email:  ricki@@rwthi3.informatik.rwth-aachen.de
d32 4
a35 1
        $Log: DATES.PAS $
d43 1
a43 1
    pversion = '$Id: DATES.PAS 1.1 1993/01/22 00:54:17 Ricki Exp $';
@


1.1
log
@Initial revision
@
text
@d1 2
a2 1
{ $Id$ }
d32 3
a34 1
        $Log$
d40 1
a40 1
    pversion = '$Id$';
@

