{$I SWITCHES.INC}
{ $Id: standard.pas 1.5 1993/04/28 23:09:41 Ricki Exp $ }
unit standard;
{ some standard procedures used every five (programming) minutes }

{
  This code is in the public domain. You may use it freely.
  There are no guarantees, either expressed or implied,
  as to either merchantability or fitness for a particular
  purpose. The author's liability is limited to the amount
  you paid for it.

  Richard Breuer

  Brunssumstrasse 6
  5100 Aachen
  Germany

  Phone:  +49/241/85605
  Fax:    +49/241/8021329
  Email:  ricki@pool.informatik.rwth-aachen.de
}

{
        $Log: standard.pas $
        Revision 1.5  1993/04/28  23:09:41  Ricki
        use conclose and stdclose in procedure stop

        Revision 1.4  1993/04/11  18:34:00  Ricki
        address change
        added output of ERRORLEVEL in case of abortion

        Revision 1.3  1993/04/05  19:56:28  Ricki
        address change

        Revision 1.2  1993/03/19  17:37:35  Ricki
        some changes in output
        new procedure getmemory

        Revision 1.1  1993/01/22  00:55:15  Ricki
        Initial revision
}

interface

  uses
    dos;

  const
    pversion = '$Id: standard.pas 1.5 1993/04/28 23:09:41 Ricki Exp $';

    stdbufsize = 16384;

  procedure header(desv,year,func: string);
  procedure revision(desn,desv: string);
  procedure err(msg: string);
  procedure interr(proc: string);
  procedure warn(msg: string);
  procedure author;
  procedure stdhelp(indent: byte);
  procedure startdbg;
  procedure stop(code: word);
  procedure report(opts,pid: string);
    {
      header    display a header containing year(s), revision desv and
                function of a RUTILS program
      revision  display design name and revision of a RUTILS program
      err       display an error message and stop(255)
      interr    display an internal-error message and stop(255)
      warn      display a warning and return to the caller
      author    display who I (Ricki) am
      stdhelp   default action for help pages (set indent & mode)
      startdbg  sets debug := true. Affects err, interr, and report
      stop      flush stdout and conout and halt(code). Display info, if
                debug is true
      report    display some information, if debug is true. Otherwise nothing
    }

  procedure txtinp(var f: text; fn: string);
  procedure txtout(var f: text; fn: string);
  procedure bininp(var f: file; fn: string);
  procedure binout(var f: file; fn: string);
    {
      txtinp    assign(f,fn) and reset it; failure --> error halt
      txtout    assign(f,fn) and rewrite it; failure --> error halt
      bininp    assign(f,fn) and reset(f,1); failure --> error halt
      binout    assign(f,fn) and rewrite(f,1); failure --> error halt
      -- THE ABOVE PROCEDURE ASSIGN NO TEXTBUFFER TO THE TEXT FILES! --
    }

  type
    revstring = string[10];

  function revno(revstr: string): revstring;
  procedure getmemory(var ptr; size: word);
    {
      revno      returns the 3.4 from '$Revision 3.4 $'
      getmemory  calls getmem w/error handling
    }

  var { readonly! }
    progname: namestr;
    progpath: pathstr;
    {
      progname  the name of the utility currently running (upper case)
      progpath  the path from where it has been started
    }

implementation

  uses
    cmscan,
    options,
    strings,
    outcon,
    outstd,
    textfmt,
    timer,
    dostools;

  var
    lprog: byte;
    starttix: longint;
    debug: boolean;

  procedure startdbg;
  begin debug := true end;

  {local} procedure getproginfo;
  var
    dummy: string;
    i: byte;
  begin
    { cmscan may not be used here due to circular unit references }
    fsplit(cmstr(0),progpath,progname,dummy);
    progpath := strupcase(progpath);
    progname := strupcase(progname);
    if (progname='') or (progname[1]=' ') then progname := '(stdin)';
    lprog := length(progname)
  end;

  procedure header(desv,year,func: string);
  begin
    findent(stdout,lprog+6+length(desv));
    fwr1teln(stdout,progname+' v'+desv+' -- '+func);
    findent(stdout,9);
    fwr1teln(stdout,'Copr (c) '+year+' Richard Breuer. '+progname+' is freeware. No warranties.');
  end;

  procedure revision(desn,desv: string);
  begin
    findent(stdout,0);
    if (desn=progname)
      then fwr1teln(stdout,'This is '+desn+' v'+desv+'R (from RUTILS 4).')
      else fwr1teln(stdout,'This is '+desn+' v'+desv
                          +'R - renamed to '+progname+' (from RUTILS 4).');
  end;

  procedure err;
  begin
    if (progname<>'') then begin
      findent(conout,lprog+9);
      fwr1teln(conout,progname+': Error: '+msg+'! Type '+progname+' -h/-H for help')
    end else begin
      findent(conout,0);
      fwr1teln(conout,'Error: '+msg+'!')
    end;
    stop(255)
  end;

  procedure interr;
  begin
    if (progname<>'') then begin
      findent(conout,lprog+2);
      fwr1teln(conout,progname+': Internal error in '+proc+'! Contact Ricki!')
    end else begin
      findent(conout,0);
      fwr1teln(conout,'Internal error in '+proc+'! Contact Ricki!')
    end;
    stop(255)
  end;

  procedure warn;
  begin
    if (progname<>'') then begin
      findent(conout,lprog+11);
      fwr1teln(conout,progname+': Warning: '+msg+'!')
    end else begin
      findent(conout,lprog+9);
      fwr1teln(conout,'Warning: '+msg+'!')
    end
  end;

  procedure author;
  begin
    findent(stdout,0);
    fwr1teln(stdout,'Author: Richard Breuer');
    fwr1teln(stdout,'        Brunssumstrasse 6');
    fwr1teln(stdout,'        5100 Aachen');
    fwr1teln(stdout,'        (after Jul 1, 1993: 52074 Aachen)');
    fwr1teln(stdout,'        Germany');
    fwr1teln(stdout,'        Europe');
    fwr1teln(stdout,'');
    fwr1teln(stdout,'Phone:  +49/241/85605');
    fwr1teln(stdout,'Fax:    +49/241/8021329');
    fwr1teln(stdout,'');
    fwr1teln(stdout,'Email:  ricki@pool.informatik.rwth-aachen.de (Preferred!)');
  end;

  procedure stdhelp(indent: byte);
  begin
    fmode(stdout,m_block);
    fwidth(stdout,79);
    findent(stdout,indent);
  end;

  function revno(revstr: string): revstring;
  var
    p: byte;
  begin
    p := pos(':',revstr);
    delete(revstr,1,p+1);
    p := pos(' ',revstr);
    revno := copy(revstr,1,p-1)
  end;

  procedure stop(code: word);
  var
    r: float;
  begin
    if (debug) then begin
      r := difftix(starttix,clocktix);
      r := r/TIX_PER_SECOND;
      if (code=0) then
        writeln(conout,'       Successful termination after ',r:0:1,' seconds')
      else
        writeln(conout,'       Abortion after ',r:0:1,' seconds, setting ERRORLEVEL to ',code);
      writeln(conout,'       Terminal free memory ',memavail,' (',maxavail,') bytes');
    end;
    flush(conout);
    flush(stdout);
    stdclose;
    conclose;
    halt(code)
  end;

  procedure txtinp(var f: text; fn: string);
  var
    code: word;
  begin
    assign(f,fn); reset(f);
    code := ioresult;
    if (code<>0) then err('Cannot open '+fn+' for reading')
  end;

  procedure txtout(var f: text; fn: string);
  var
    code: word;
  begin
    assign(f,fn); rewrite(f);
    code := ioresult;
    if (code<>0) then err('Cannot open '+fn+' for writing')
  end;

  procedure bininp(var f: file; fn: string);
  var
    code: word;
  begin
    assign(f,fn); reset(f,1);
    code := ioresult;
    if (code<>0) then err('Cannot open '+fn+' for reading')
  end;

  procedure binout(var f: file; fn: string);
  var
    code: word;
  begin
    assign(f,fn); rewrite(f,1);
    code := ioresult;
    if (code<>0) then err('Cannot open '+fn+' for writing')
  end;

  procedure report(opts,pid: string);
  var
    i,nopt,nnam: byte;
  begin if (debug) then begin
    { get nopt and nnam }
    parse(opts,nopt,nnam);

    writeln(conout,'Debug: EXE = ',progpath+progname);
    writeln(conout,'       ',pid);
    writeln(conout,'       Possible options are ',opts);
    writeln(conout,'       Params = ',nopt,' Option(s), ',nnam,' Name(s)');

    for i:=1 to nopt do begin
      write(conout,'       Option #1 = -',getopt(i),', Argument is ');
      if (optarg='') then writeln(conout,'empty')
                     else writeln(conout,'"',printable(optarg),'"')
    end;

    for i:=1 to nnam do begin
      writeln(conout,'       Name #1 = "',printable(getname(i)),'"')
    end;

    if (InpRedirected) then writeln(conout,'       Input is redirected')
                       else writeln(conout,'       Input is not redirected');

    if (OutToNUL)           then writeln(conout,'       Output is redirected to NUL')
    else if (OutRedirected) then writeln(conout,'       Output is redirected')
                            else writeln(conout,'       Output is not redirected');

    writeln(conout,'       Initial free memory ',memavail,' (',maxavail,') bytes');
  end end;

  procedure getmemory(var ptr; size: word);
  var
    q: pointer absolute ptr;
  begin
    if (maxavail<size) then err('Memory overflow')
                       else getmem(q,size)
  end;

begin
  { initialization }
  starttix := clocktix;
  debug := false;
  { set filemode so that reset opens for reading only }
  system.filemode := 0;
  { get name & path of program from the command line }
  getproginfo
end.
