{ $Id$ }
program cd;
{ Replacement for DOS's CD }

{
  This code is in the public domain. You may use it freely.
  There are no guarantees, either expressed or implied,
  as to either merchantability or fitness for a particular
  purpose. The author's liability is limited to the amount
  you paid for it.

  Richard Breuer

  Brunssumstrasse 6
  5100 Aachen
  Germany

  Phone:  +49/241/85605
  Fax:    +49/241/8021329
  Email:  ricki@rwthi3.informatik.rwth-aachen.de
}

{
        $Log$
}

uses
  files,
  standard,
  strings,
  outstd,
  outcon,
  cmdline,
  dos;

const
  func = 'Replacement for DOS''s CD';
  year = '1992,1993';
  desn = 'CD/2';
  opts = 'hHlrsv*'; ropt = opts;

var
  debug: boolean;
  opt_r,opt_s,opt_l: boolean;
  nopt,nnam: byte;
  excode: word;

  path: string;
  lpath: byte;
  gnnam: byte;

  stkfilename,stktempname: pathstr;
  stkfile,stktemp: text;

  {$i cd.i1}
  {$i cd.i2}

  procedure help1;
  begin
    header(year,func);
    cdi1;
    stop(1)
  end;

  procedure help2;
  begin
    header(year,func);
    cdi2;
    stop(1)
  end;

  procedure version;
  begin
    header(year,func);
    writeln;
    revision(desn,'$Revision: 4.3 $');
    writeln;
    author;
    stop(1)
  end;

  procedure changedir(s: string);
  { / --> \; kill trailing \'s, if necessary; call chdir }
  label
    ok;
  var
    ls: byte;
  begin
    substall('/','\',s);

    ls := length(s);
    while true do begin
      if (s[ls]<>'\') then goto ok;
      if (ls=1) then goto ok;
      if (s[ls-1]=':') then goto ok;
      dec(ls)
    end;

    ok:
    s[0] := chr(ls);
    chdir(s)
  end;

  {$f+} procedure pre_op(nopt: byte); {$f-}
  begin
    opt_r := false; opt_s := false; opt_l := false;
    excode := 0;
    stkfilename := 'C:\'+progname+'.STK';
    debug := false
  end;

  {$f+} procedure proc_op(opt: char; optarg: namestring); {$f-}
  begin
    case opt of
      'l': opt_l := true;
      'r': opt_r := true;
      's': opt_s := true;

      #0 : begin
        chdir('\');
        if (ioresult<>0) then err(progname+' fails for "\"')
                         else stop(0)
      end;
      'h': help1;
      'H': help2;
      'v': version;
      '*': debug := true;
    end
  end;

  {$f+} procedure post_op; {$f-}
  begin
    if (opt_r and opt_s) then
      err('-r and -s are mutually exclusive');
    if (debug) then report
  end;

  {$f+} procedure pre_fn(nnam: byte); {$f-}
  begin
    if (nnam>1) then
      err('Too many parameters');
    gnnam := nnam;

    { determine the cases where sth. must be added to CD.STK }
    if (opt_r) or (opt_s) or ( (opt_l) and (nnam>0) ) then begin
      stktempname := tmpname('C:\');

      if (debug) then begin
        writeln(conout,'       Name of stackfile = ',stkfilename);
        writeln(conout,'       Name of temp file = ',stktempname);
      end;

      { open/prepare stkfile for reading }
      {$i-} assign(stkfile,stkfilename); reset(stkfile); {$i-}
      { if not found, create an empty stkfile }
      if (ioresult<>0) then begin
        {$i-} assign(stkfile,stkfilename); rewrite(stkfile); {$i-}
        if (ioresult<>0) then
          err('Cannot create "'+stkfilename+'"');
        close(stkfile);
        {$i-} assign(stkfile,stkfilename); reset(stkfile) {$i-}
      end;

      { prepare stktemp for writing }
      txtout(stktemp,stktempname);

      { opt_s --> push current dir on stack }
      if (opt_s) then begin
        getdir(0,path);
        writeln(stktemp,path);
        while (not eof(stkfile)) do begin
          readln(stkfile,path);
          writeln(stktemp,path)
        end
      end

      { opt_r --> read the first dir and CD to it }
      else if (opt_r) then begin
        if (not eof(stkfile)) then begin
          readln(stkfile,path);
          changedir(path);
          if (ioresult<>0) then begin
            writeln(conout,progname,' fails for "',path,'"!');
            excode := 255
          end
        end
      end

    end
  end;

  {$f+} procedure proc_fn(var s: searchrec; p: pathstr); {$f-}
  begin
    if (s.attr and directory=0) then begin
      { a non-directory parameter makes no sense }
      writeln(conout,'"',p+s.name,'" is no directory!');
      excode := 255;
      exit
    end;

    if (opt_r) or (opt_l) then
      { push p+sr.name on the stack and copy the rest of the stack }
      writeln(stktemp,p+s.name)

    else begin
      { CD to p+sr.name }
      changedir(p+s.name);
      if (ioresult<>0) then begin
        writeln(conout,progname,' fails for "',p+s.name,'"!');
        excode := 255
      end
    end
  end;

  {$f+} procedure post_fn; {$f-}
  var
    l: word;
  begin
    { if necessary: close the files and rename temp to file }
    if (opt_r or opt_s or ( (opt_l) and (gnnam>0) )) then begin
      { copy to temp, if necessary }
      if (opt_r) or ( (opt_l) and (gnnam>0) ) then
        while (not eof(stkfile)) do begin
        readln(stkfile,path);
        writeln(stktemp,path)
      end;

      close(stkfile);
      close(stktemp);

      erase(stkfile);
      rename(stktemp,stkfilename)
    end;

    if (opt_l) then begin
      { display stkfile }
      txtinp(stkfile,stkfilename);
      l := 0;
      while (not eof(stkfile)) do begin
        inc(l);
        readln(stkfile,path);
        if (l=1) then writeln(stdout,'The directory stack contains:');
        writeln(stdout,l,' - ',path)
      end;
      close(stkfile);
      if (l=0) then writeln(stdout,'The directory stack is empty.')
    end
  end;

begin
  process(ropt,
          pre_op,proc_op,post_op,
          pre_fn,proc_fn,post_fn,
          m_quiet or m_dir);
  stop(excode)
end.
