head	1.7;
access;
symbols
	rutils40:1.5;
locks;
comment	@        @;


1.7
date	93.04.17.13.21.13;	author Ricki;	state Exp;
branches;
next	1.6;

1.6
date	93.03.31.19.24.17;	author Ricki;	state Exp;
branches;
next	1.5;

1.5
date	93.03.18.11.47.22;	author Ricki;	state Rel;
branches;
next	1.4;

1.4
date	93.02.20.21.13.22;	author Ricki;	state Exp;
branches;
next	1.3;

1.3
date	93.02.10.22.13.25;	author Ricki;	state Exp;
branches;
next	1.2;

1.2
date	93.02.07.16.49.57;	author Ricki;	state Exp;
branches;
next	1.1;

1.1
date	93.02.03.17.01.36;	author Ricki;	state Exp;
branches;
next	;


desc
@Disk cataloger
@

1.7
log
@replace tabs by spaces in comments/descriptions and volume
  names to be able to use tabs as delimiters in the catalog file
@
text
@{$I SWITCHES.INC}
{ $Id: dcat.pas 1.6 1993/03/31 19:24:17 Ricki Exp $ }
program dcat;
{ Disk Cataloger }

{
  This code is in the public domain. You may use it freely.
  There are no guarantees, either expressed or implied,
  as to either merchantability or fitness for a particular
  purpose. The author's liability is limited to the amount
  you paid for it.

  Richard Breuer

  Brunssumstrasse 6
  5100 Aachen
  Germany

  Phone:  +49/241/85605
  Fax:    +49/241/8021329
  Email:  ricki@@pool.informatik.rwth-aachen.de


  Header line ::=
    "Volume" TAB volume TAB yyyy/mm/dd TAB time TAB size TAB free TAB used
  Entry line  ::=
    pathname TAB [comment] TAB size TAB time TAB attr
}

{
        $Log: dcat.pas $
        Revision 1.6  1993/03/31  19:24:17  Ricki
        update to new email address

        Revision 1.5  1993/03/18  11:47:22  Ricki
        correction: some writes did not use the stdout file

        Revision 1.4  1993/02/20  21:13:22  Ricki
        bugfix: archive comment was overwritten by non-existing
          DESCRIPT.ION comment

        Revision 1.3  1993/02/10  22:13:25  Ricki
        modified -v page

        Revision 1.2  1993/02/07  16:49:57  Ricki
        new command alias: 'dir' = 'list'
        avoided call of 'fileinfo'

        Revision 1.1  1993/02/03  17:01:36  Ricki
        Initial revision
}

{
        Conditional defines
          DEBUG     Enables debugging mode. Sets debug := true
}

uses
  comments,
  descript,
  standard,
  files,
  strings,
  numbers,
  dates,
  keyboard,
  options,
  outcon,
  outstd,
  cmdline,
  dos;

const
  pversion = '$Id: dcat.pas 1.6 1993/03/31 19:24:17 Ricki Exp $';

  func = 'Disk Cataloger';
  year = '1993';
  desn = 'DCAT';
  desv = '1.0';
  opts = 'acf:hHnqv*'; ropt = opts;

  headerTxt = 'Volume'; 
  trailerTxt = '';
  volumeLen = 11;

  default_drive = 'A';

  maxnames = 512;

type
  strptr = ^string;

  namerec = record
    name: strptr;
    comment: strptr;   { nil, if none }
    size,time: longint;
    attr: word
  end;
  namebuftype = array [1..maxnames] of namerec;

  diskrec = record
    vol: namexstr;
    size,free,used,time: longint
  end;

  actiontype = (a_none,
                a_add,a_update,a_multi,a_cat,a_volumes,a_find,a_remove,
                a_removx,a_dir,a_list,a_sum,a_help,
                a_ambig);

var
  debug: boolean;
  nopt,nnam: byte;
  fbuf,obuf: pointer;

  dcfile,dctemp: text;
  temp_assigned: boolean;
  dcfilename,dctempname: pathstr;
  tmpbuf: namebuftype;
  i,ntmpbuf: integer;
  usedsum: longint;
  filesum: integer;
  dpath: pathstr;
  dfile: text;
  c: char;

  opta,optc,optn,optf,optq: boolean;

  {$i dcat.i1}
  {$i dcat.i2}

  {$i-} { turn off I/O checking for all procedures }

  { ---------------------------------------------------------------------- }
  {                        General purpose procedures                      }
  { ---------------------------------------------------------------------- }

  procedure help1;
  begin
    header(desv,year,func);
    dcati1;
    stop(1)
  end;

  procedure help2;
  begin
    header(desv,year,func);
    dcati2;
    stop(1)
  end;

  procedure version;
  begin
    header(desv,year,func);
    writeln(stdout);
    revision(desn,desv);
    writeln(stdout);
    author;
    stop(1)
  end;

  procedure err(s: string);
  begin
    if (temp_assigned) then begin
      erase(dctemp);
      if (ioresult<>0) then ;
    end;
    writeln(stdout);
    standard.err(s)
  end;

  function checkdrive(s: string): boolean;
  begin
    checkdrive := (s='A') or (s='A:') or (s='B') or (s='B:')
               or (s='a') or (s='a:') or (s='b') or (s='b:')
  end;

  function today: longint;
  { return the packed date of today }
  var
    d: datetime;
    dummy: word;
    v: longint;
  begin with d do begin
    getdate(year,month,day,dummy);
    gettime(hour,min,sec,dummy);
    packtime(d,v);
    today := v
  end end;

  function waste(disk: diskrec): real;
  { calculate the wasted disk space in % of the disk size }
  begin with disk do begin
    if (size<>0) then waste := (size-free-used)*100.0/size
                 else waste := 0;
  end end;

  { ---------------------------------------------------------------------- }
  {                    Handling the floppy header string                   }
  { ---------------------------------------------------------------------- }

  function Hgetvol(s: string): namexstr;
  { extract the volume from a floppy header string. Trailing blanks are
    NOT REMOVED! }
  var
    p: byte;
  begin
    p := pos(#9,s); delete(s,1,p); { skip 'Volume' }
    p := pos(#9,s); if (p=0) then p := 1+length(s);
    Hgetvol := copy(s,1,p-1)
  end;

  procedure Hgetdiskrec(s: string; var d: diskrec);
  { extract the disk info from a floppy header string }
  var
    p: byte;
    v: longint;
  begin with d do begin
    size := 0;
    free := 0;
    used := 0;
    time := 0;
    p := pos(#9,s); delete(s,1,p); { skip 'Volume' }
    p := pos(#9,s); if (p=0) then p := 1+length(s);
      vol := copy(s,1,p-1); delete(s,1,p);
    p := pos(#9,s); if (p=0) then p := 1+length(s);
      if (p>0) and (ival(copy(s,1,p-1),v)) then time := v;
      delete(s,1,p);
    p := pos(#9,s); if (p=0) then p := 1+length(s);
      if (p>0) and (ival(copy(s,1,p-1),v)) then size := v;
      delete(s,1,p);
    p := pos(#9,s); if (p=0) then p := 1+length(s);
      if (p>0) and (ival(copy(s,1,p-1),v)) then free := v;
      delete(s,1,p);
    p := pos(#9,s); if (p=0) then p := 1+length(s);
      if (p>0) and (ival(copy(s,1,p-1),v)) then used := v
  end end;

  { ---------------------------------------------------------------------- }
  {                         Handling name entries                          }
  { ---------------------------------------------------------------------- }

  procedure Egetnamerec(s: string; var n: namerec);
  { extract all info from an entry string }
  var
    p: byte;
    v: longint;
  begin with n do begin
    size := 0;
    time := 0;
    attr := 0;

    p := pos(#9,s); if (p=0) then p := 1+length(s);
      getmemory(name,p); name^ := copy(s,1,p-1); delete(s,1,p);

    p := pos(#9,s); if (p=0) then p := 1+length(s);
    if (p>1) then begin
      getmemory(comment,p); comment^ := copy(s,1,p-1)
    end else
      comment := nil;
    delete(s,1,p);

    p := pos(#9,s); if (p=0) then p := 1+length(s);
      if (p>0) and (ival(copy(s,1,p-1),v)) then size := v;
      delete(s,1,p);

    p := pos(#9,s); if (p=0) then p := 1+length(s);
      if (p>0) and (ival(copy(s,1,p-1),v)) then time := v;
      delete(s,1,p);

    p := pos(#9,s); if (p=0) then p := 1+length(s);
      if (p>0) and (ival(copy(s,1,p-1),v)) then attr := v
  end end;

  function Egetname(s: string): string;
  { extract a pathname from an entry string }
  begin
    Egetname := copy(s,1,pos(#9,s)-1)
  end;

  { ---------------------------------------------------------------------- }
  {                    Writing information to catalogs                     }
  { ---------------------------------------------------------------------- }

  procedure Wheader(var outfile: text; disk: diskrec);
  { write the header line for a volume to outfile }
  begin with disk do begin
    writeln(outfile,headerTxt,
                    #9,
                    copy(vol+blanks,1,volumeLen+1),
                    #9,
                    time,
                    #9,
                    size,
                    #9,
                    free,
                    #9,
                    used)
  end end;

  procedure Wentry(var outfile: text; e: namerec);
  { write the file name line for a volume to outfile }
  begin
    if (e.comment<>nil) then
      writeln(outfile,e.name^,
                      #9,
                      e.comment^,
                      #9,
                      e.size,
                      #9,
                      e.time,
                      #9,
                      e.attr)
    else
      writeln(outfile,e.name^,
                      #9,
                      #9,
                      e.size,
                      #9,
                      e.time,
                      #9,
                      e.attr)
  end;

  { ---------------------------------------------------------------------- }
  {                    Showing information to stdout                       }
  { ---------------------------------------------------------------------- }

  {
    HEADER:
    Volume VVVVVVVVVVV   MM/DD/YYYY    fffffff free    uuuuuuu used    sssssss size

    DISK:
    VVVVVVVVVVV  MM/DD/YYYY  NNN  FFFFFFF  UUUUUUU  SSSSSSS

    ENTRY1 (dc dir <vol>):
    NNNNNNNN.NNN  MM/DD/YYYY  SSSSSSS  SAHR  C...(38)

    ENTRY2 (dc find <str>):
    NNNNNNNN.NNN  VVVVVVVVVVV  C...(52)

    ENTRY3 (dc find <str> -a):
    ...NNNNNNNN.NNN  VVVVVVVVVVV  MM/DD/YYYY  SSSSSSS  SAHR  C...

    ENTRY4 (dc dir <vol> -a):
    ...NNNNNNNN.NNN  MM/DD/YYYY  SSSSSSS  SAHR  C...
  }

  procedure Sheader(disk: diskrec);
  { Volume VVVVVVVVVVV   MM/DD/YYYY    fffffff free    uuuuuuu used    sssssss size }
  var
    dt: datetime;
  begin with disk do begin
    unpacktime(time,dt);
    writeln(stdout,'Volume ',
                   copy(vol+blanks,1,volumeLen),
                   '   ',
                   date2str(dt.day,dt.month,dt.year),
                   '    ',
                   free:7,
                   ' free    ',
                   used:7,
                   ' used    ',
                   size:7,
                   ' size')
  end end;

  procedure Sdisk(disk: diskrec; nfiles: integer);
  { VVVVVVVVVVV  MM/DD/YYYY  NNN  FFFFFFF  UUUUUUU  SSSSSSS WWW.WW%}
  var
    dt: datetime;
  begin with disk do begin
    unpacktime(time,dt);
    writeln(stdout,copy(vol+blanks,1,volumeLen),
                   '  ',
                   date2str(dt.day,dt.month,dt.year),
                   '  ',
                   nfiles:3,
                   '  ',
                   free:7,
                   '  ',
                   used:7,
                   '  ',
                   size:7,
                   ' ',
                   waste(disk):6:2,
                   '%')
  end end;

  procedure Sentry1(e: namerec; vol: namexstr);
  { NNNNNNNN.NNN  MM/DD/YYYY  SSSSSSS  SAHR  C...(38) }
  var
    dt: datetime;
  begin with e do begin
    unpacktime(time,dt);
    write(stdout,copy(filenamex(name^)+blanks,1,12),
                 '  ',
                 date2str(dt.day,dt.month,dt.year),
                 '  ',
                 size:7,
                 '  ',
                 attr2str(attr));
    if (comment=nil) then writeln(stdout)
                     else writeln(stdout,'  ',copy(comment^+blanks,1,38))
  end end;

  procedure Sentry2(e: namerec; vol: namexstr);
  { NNNNNNNN.NNN  VVVVVVVVVVV  C...(52) }
  begin with e do begin
    write(stdout,copy(filenamex(name^)+blanks,1,12),
                 '  ',
                 copy(vol+blanks,1,11));
    if (comment=nil) then writeln(stdout)
                     else writeln(stdout,'  ',copy(comment^+blanks,1,52))
  end end;

  procedure Sentry3(e: namerec; vol: namexstr);
  { ...NNNNNNNN.NNN  VVVVVVVVVVV  MM/DD/YYYY  SSSSSSS  SAHR  C... }
  var
    dt: datetime;
    n: string;
  begin with e do begin
    unpacktime(time,dt);
    n := name^;
    if (length(n)<12) then n := copy(n+blanks,1,12);
    write(stdout,n,
                 '  ',
                 copy(vol+blanks,1,11),
                 '  ',
                 date2str(dt.day,dt.month,dt.year),
                 '  ',
                 size:7,
                 '  ',
                 attr2str(attr));
    if (comment=nil) then writeln(stdout)
                     else writeln(stdout,'  ',comment^)
  end end;

  procedure Sentry4(e: namerec; vol: namexstr);
  { ...NNNNNNNN.NNN  MM/DD/YYYY  SSSSSSS  SAHR  C... }
  var
    dt: datetime;
    n: string;
  begin with e do begin
    unpacktime(time,dt);
    n := name^;
    if (length(n)<12) then n := copy(n+blanks,1,12);
    write(stdout,n,
                 '  ',
                 date2str(dt.day,dt.month,dt.year),
                 '  ',
                 size:7,
                 '  ',
                 attr2str(attr));
    if (comment=nil) then writeln(stdout)
                     else writeln(stdout,'  ',comment^)
  end end;

  { some headers }
  procedure Hentry1;
  begin writeln(stdout,'Name          Date           Size  Attr  Comment')
  end;
  procedure Hdisk;
  begin writeln(stdout,'Volume       Date        Files   Free     Used     Size   Waste')
  end;
  procedure Hentry2;
  begin writeln(stdout,'Name          Volume       Comment')
  end;
  procedure Hentry3;
  begin writeln(stdout,'Name...       Volume       Date           Size  Attr  Comment...')
  end;
  procedure Hentry4;
  begin writeln(stdout,'Name...       Date           Size  Attr  Comment...')
  end;
  procedure Hfatline;
  begin writeln(stdout,'===============================================================================')
  end;
  procedure Hthinline;
  begin writeln(stdout,'-------------------------------------------------------------------------------')
  end;

  { ---------------------------------------------------------------------- }
  {                         Handling catalog files                         }
  { ---------------------------------------------------------------------- }

  procedure update(var flopfile,outfile: text; disk: diskrec;
                   var code: word);
  { add the contents of tmpbuf to flopfile and write the result on outfile.
    vol is the label of the diskette. Code=1, if reset(flopfile) failed,
    Code=2, if rewrite(outfile) failed; code>0 for other errors }
  var
    s: string;
    i: word;
    found: boolean;
    cmp: namexstr;
  begin
    reset(flopfile);
    if (ioresult<>0) then code := 1
                     else code := 0;
    if (code=0) then begin
      rewrite(outfile);
      if (ioresult<>0) then code := 2
    end;
    if (code<>0) then exit;

    if (not optq) then write(stdout,'Updating catalog file '+dcfilename);

    { read flopfile and copy to outfile until a label greater or eqal vol is
      reached }
    found := false;
    cmp := '';

    while (not eof(flopfile)) and (not found) do begin
      readln(flopfile,s);
      cmp := Hgetvol(s);
      found := (cmp>=disk.vol);

      { if vol is not yet reached, transfer s and the following lines
        until trailerTxt to outfile }
      if (not found) then begin
        writeln(outfile,s);
        while (not eof(flopfile)) and (s<>trailerTxt) do begin
          readln(flopfile,s);
          writeln(outfile,s)
        end
      end
    end;

    Wheader(outfile,disk);
    { write the contents of tmpbuf + trailerTxt to outfile and dispose
      tmpbuf }
    for i:=1 to ntmpbuf do begin
      Wentry(outfile,tmpbuf[i]);
      freemem(tmpbuf[i].name,1+length(tmpbuf[i].name^));
      if (tmpbuf[i].comment<>nil) then
        freemem(tmpbuf[i].comment,1+length(tmpbuf[i].comment^))
    end;
    writeln(outfile,trailerTxt);

    { if vol=cmp, skip the floppy in flopfile }
    if (disk.vol=cmp) then begin
      while (not eof(flopfile)) and (s<>trailerTxt) do
        readln(flopfile,s);
      if (not eof(flopfile)) then
        readln(flopfile,s)
    end;

    { copy the rest of flopfile to outfile }
    if (not eof(flopfile)) then
      writeln(outfile,s);
    while (not eof(flopfile)) do begin
      readln(flopfile,s);
      writeln(outfile,s)
    end;

    if (not optq) then writeln(stdout,' Done.')

  end;

  procedure remove(var flopfile,outfile: text; vol: namexstr; var code: word);
  { remove all disks vol* from the flopfile; for parameters see update }
  var
    s: string;
    cmp: namexstr;
  begin
    reset(flopfile);
    if (ioresult<>0) then code := 1
                     else code := 0;
    if (code=0) then begin
      rewrite(outfile);
      if (ioresult<>0) then code := 2
    end;
    if (code<>0) then exit;

    if (not optq) then writeln(stdout,'Searching catalog file '+dcfilename);

    { read flopfile and copy to outfile until a label greater or eqal vol is
      reached }
    vol := strupcase(vol);
    
    while (not eof(flopfile)) do begin
      readln(flopfile,s);
      cmp := strupcase(Hgetvol(s));

      { if vol*=cmp, skip this disk }
      if (copy(cmp,1,length(vol))=vol) then begin

        if (not optq) then writeln(stdout,'  Deleting Volume ',cmp);
        while (not eof(flopfile)) and (s<>trailerTxt) do readln(flopfile,s);

      end else begin

        { copy the disk to outfile }
        writeln(outfile,s);
        while (not eof(flopfile)) and (s<>trailerTxt) do begin
          readln(flopfile,s);
          writeln(outfile,s)
        end
      end
    end;

  end;

  procedure voldir(var flopfile: text; vol: namexstr; var code: word);
  { display the contents of vol* }
  var
    s: string;
    cmp: namexstr;
    disk: diskrec;
    nr: namerec;
  begin
    reset(flopfile);
    if (ioresult<>0) then code := 1
                     else code := 0;
    if (code<>0) then exit;

    if (not optq) then begin
      writeln(stdout,'Searching catalog file '+dcfilename);
      writeln(stdout)
    end;

    vol := strupcase(vol);
    while (not eof(flopfile)) do begin
      readln(flopfile,s);
      cmp := strupcase(Hgetvol(s));

      { if vol*>cmp, skip this disk }
      if (copy(cmp,1,length(vol))<vol) then begin
        while (not eof(flopfile)) and (s<>trailerTxt) do readln(flopfile,s)
      end

      { if vol*<cmp, exit }
      else if (copy(cmp,1,length(vol))>vol) then exit

      else begin
        { list the disk on stdout }
        Hgetdiskrec(s,disk);

        if (not optq) then begin
          HFatline;
          Sheader(disk);
          HFatline;
          if (opta) then Hentry4 else Hentry1;
          Hthinline
        end else Sheader(disk);

        while (not eof(flopfile)) and (s<>trailerTxt) do begin
          readln(flopfile,s);
          if (s<>trailerTxt) then begin
            Egetnamerec(s,nr);
            if (opta) then Sentry4(nr,cmp)
                      else Sentry1(nr,cmp);
            freemem(nr.name,1+length(nr.name^));
            if (nr.comment<>nil) then
              freemem(nr.comment,1+length(nr.comment^))
          end
        end;
        writeln(stdout)
      end
    end
  end;

  procedure cat(var flopfile: text; vol: namexstr; var code: word);
  { display a list of all disks }
  var
    s: string;
    cmp: namexstr;
    disk: diskrec;
    nfiles: integer;
  begin
    reset(flopfile);
    if (ioresult<>0) then code := 1
                     else code := 0;
    if (code<>0) then exit;

    if (not optq) then begin
      writeln(stdout,'Searching catalog file '+dcfilename);
      writeln(stdout)
    end;

    if (not optq) then begin
      HFatline;
      Hdisk;
      Hthinline;
    end;

    vol := strupcase(vol);
    while (not eof(flopfile)) do begin
      readln(flopfile,s);
      cmp := strupcase(Hgetvol(s));

      { if vol*>cmp, skip this disk }
      if (copy(cmp,1,length(vol))<vol) then begin
        while (not eof(flopfile)) and (s<>trailerTxt) do readln(flopfile,s)
      end

      { if vol*<cmp, exit }
      else if (copy(cmp,1,length(vol))>vol) then exit

      else begin
        { list the disk on stdout }
        Hgetdiskrec(s,disk);

        nfiles := -1;
        while (not eof(flopfile)) and (s<>trailerTxt) do begin
          inc(nfiles);
          readln(flopfile,s)
        end;

        Sdisk(disk,nfiles);
      end
    end
  end;

  procedure findstr(var flopfile: text; fs: string; var code: word);
  { find *str* in the filename or the comment }
  var
    s: string;
    cmp: namexstr;
    nr: namerec;
  begin
    reset(flopfile);
    if (ioresult<>0) then code := 1
                     else code := 0;
    if (code<>0) then exit;

    if (not optq) then begin
      writeln(stdout,'Searching catalog file '+dcfilename);
      writeln(stdout)
    end;

    fs := strupcase(fs);

    if (not optq) then begin
      Hfatline;
      if (opta) then Hentry3 else Hentry2;
      Hthinline
    end;

    while (not eof(flopfile)) do begin
      readln(flopfile,s);
      cmp := Hgetvol(s);

      while (not eof(flopfile)) and (s<>trailerTxt) do begin
        readln(flopfile,s);
        if (pos(fs,strupcase(s))<>0) then begin
          Egetnamerec(s,nr);
          if (opta) then Sentry3(nr,cmp)
                    else Sentry2(nr,cmp);
          freemem(nr.name,1+length(nr.name^));
          if (nr.comment<>nil) then
            freemem(nr.comment,1+length(nr.comment^))
        end
      end
    end
  end;

  procedure summarize(var flopfile: text; var code: word);
  { display overall statistics about the catalog }
  var
    s: string;
    cmp: namexstr;
    totaldisks,totalfiles: longint;
    curfiles,minfiles,maxfiles: longint;
    maxwaste,x: real;
    disk: diskrec;
  begin
    reset(flopfile);
    if (ioresult<>0) then code := 1
                     else code := 0;
    if (code<>0) then exit;

    if (not optq) then begin
      writeln(stdout,'Scanning catalog file '+dcfilename);
      writeln(stdout)
    end;

    totaldisks := 0;
    totalfiles := 0;
    minfiles := maxnames;
    maxfiles := 0;
    maxwaste := 0.0;

    while (not eof(flopfile)) do begin
      readln(flopfile,s);

      Hgetdiskrec(s,disk);
      x := waste(disk);
      if (x>maxwaste) then maxwaste := x;

      inc(totaldisks);
      curfiles := 0;
      while (not eof(flopfile)) and (s<>trailerTxt) do begin
        inc(totalfiles);
        inc(curfiles);
        readln(flopfile,s);
      end;

      if (curfiles>maxfiles) then maxfiles := curfiles;
      if (curfiles<minfiles) then minfiles := curfiles
    end;

    writeln(stdout,'The catalog contains ',totalfiles,' files on ',totaldisks,' diskettes.');
    if (totalfiles>0) then begin
      x := totalfiles / totaldisks;
      writeln(stdout,'This means an average of ',x:0:2,' files per volume.');
      writeln(stdout,'The number of files per volume varies from ',minfiles-1,
                                                            ' to ',maxfiles-1,'.');
      writeln(stdout,'The maximum waste is',maxwaste:6:2,'%.');
    end

  end;

  { ---------------------------------------------------------------------- }
  {                        Disk access procedures                          }
  { ---------------------------------------------------------------------- }

  {$f+} procedure addtobuf(var sr: searchrec; p: pathstr); {$f-}
  { this procedure will be called for each file on the disk }
  var
    ln: byte;
    mlc: integer;
    f: file;
    c,c2: string;
    ftype: supported;
  begin
    if (not optq) then write(stdout,'  '+p+sr.name);
    ln := 2+length(p)+length(sr.name);

    { store the file name without d:\ }
    inc(ntmpbuf);
    if (ntmpbuf>maxnames) then begin
      writeln(stdout);
      err('Too many files')
    end;

    with tmpbuf[ntmpbuf] do begin
      getmemory(name,length(p)+length(sr.name)-2);
      name^ := copy(p,4,255)+sr.name;

      c := '';

      { Archive comment? }
      if (not optn) and ( (optc) or hascomment(fileext(sr.name)) ) then begin
        { try to read a comment }
        assign(f,p+sr.name); reset(f,1);
        if (ioresult=0) then begin
          getcomment(f,ftype,c);
          close(f)
        end;
      end;

      { DESCRIPT.ION overrides archive comments }
      if (not optn) then begin
        { open current DESCRIPT.ION, if not already done }
        if (dpath<>p) then begin
          if (dpath<>#0) then close(dfile);
          dpath := p;
          assign(dfile,dpath+'DESCRIPT.ION')
        end;
        getdesc(dfile,sr.name,c2);
        if (c2<>'') then c := c2
      end;

      { store the comment, if one's there }
      if (c<>'') then begin
        { replace all tabs in the comment by spaces in order to be able
	  to use tabs as delimiters in DCAT.DAT }
        c := detab(c,8);
        getmemory(comment,length(c)+1);
        comment^ := c
      end else
        comment := nil;

      { get further info }
      with tmpbuf[ntmpbuf] do begin
        size := sr.size;
        time := sr.time;
        attr := sr.attr
      end;

      { increment global values }
      inc(filesum);
      inc(usedsum,size);

      { report }
      if (not optq) then with tmpbuf[ntmpbuf] do begin
        if (comment=nil) then writeln(stdout)
        else begin
          if (ln<18) then begin write(stdout,' ':18-ln); ln := 18 end;
          writeln(stdout,'  >',copy(comment^,1,79-ln-3))
        end
      end
    end
  end;

  procedure readdisk(drive: char; var disk: diskrec; var code: word);
  { read the disk in drive: and transfer it to disk & tmpbuf. vol is
    filled with blanks to its length }
  var
    bcode: byte;
  begin with disk do begin
    if (not optq) then write(stdout,'Scanning Disk '+drive+':');

    { get disk information }
    diskinfo(drive,vol,size,free,code);
    if (code<>0) or (vol='') or (size=0) then exit;

    { replace tabs in the volume by spaces in order to be able to use
      tabs as delimiters in DCAT.DAT }
    vol := detab(vol,8);

    { report }
    if (not optq) then
      writeln(stdout,'  Volume: ',vol,'  Free/Size: ',free,'/',size);

    { loop over all files }
    ntmpbuf := 0;
    usedsum := 0; filesum := 0; { incremented in addtobuf }
    dpath := #0;
    forallrec(drive+':\','*.*',archive or readonly or sysfile,addtobuf,bcode);
    if (dpath<>#0) then close(dfile);
    if (ioresult=0) then {doesn't matter};
    code := bcode;
    used := usedsum;

    if (not optq) then begin
      writeln(stdout);
      writeln(stdout,'  Total: ',filesum,' Files with ',usedsum,' Bytes.');
      writeln(stdout)
    end;

    { fill vol with blanks and set the date of cataloging }
    vol := copy(vol+blanks,1,volumeLen+1);
    time := today
  end end;

  { ---------------------------------------------------------------------- }
  {                        Command line handling                           }
  { ---------------------------------------------------------------------- }

  procedure pre_op;
  begin
    opta := false; optc := false;
    optf := false; optq := false;
    optn := false;
{$IFDEF DEBUG}
    debug := true
{$ELSE}
    debug := false
{$ENDIF}
  end;

  procedure proc_op;
  var
    i: byte;
  begin
    parse(ropt,nopt,nnam);
    if (nopt+nnam=0) then help1;
    for i:=1 to nopt do begin
      case getopt(i) of
        'a': opta := true;
        'c': optc := true;
        'f': begin
          optf := true;
          if (optarg='') then err('Empty catalog filename')
                         else dcfilename := optarg
        end;
        'n': optn := true;
        'q': optq := true;

        'h': help1;
        'H': help2;
        'v': version;
        '*': debug := true
      end
    end
  end;

  procedure post_op;
  begin
    if (debug) then startdbg;
    report(ropt,pversion);

    if (nnam=0) then err('Nothing to do')
  end;

  function find_command(s: string): actiontype;
  type
    acstr = string[6];
  const
    ac: array [actiontype] of acstr =
      (#0,
          'ADD','UPDATE','MULTI','CAT','VOLUMES','FIND',
          'REMOVE','REMOVx','DIR','LIST','SUM','HELP',
       #0);
  var
    idx,i: actiontype;
  begin
    s := strupcase(s);
    idx := a_none;
    for i:=succ(a_none) to pred(a_ambig) do begin
      if (copy(ac[i],1,length(s))=s) then begin
        if (idx<>a_none) then begin find_command := a_ambig; exit end;
        idx := i
      end
    end;
    find_command := idx
  end;

  { ---------------------------------------------------------------------- }
  {                        Frames for the catalog procs                    }
  { ---------------------------------------------------------------------- }

  procedure do_add(drive: char);
  var
    disk: diskrec;
    code: word;
    vol: namexstr;
  begin
    txtinp(dcfile,dcfilename);
    txtout(dctemp,dctempname); temp_assigned := true;

    settextbuf(dcfile,fbuf^,stdbufsize);
    settextbuf(dctemp,obuf^,stdbufsize);

    readdisk(drive,disk,code);
    if (code<>0) then err('Cannot read label of disk in drive '+drive+':');

    update(dcfile,dctemp,disk,code);
    if (code=1) then err('Cannot open '+dcfilename+' for reading');
    if (code=2) then err('Cannot open '+dctempname+' for writing');

    close(dcfile);
    close(dctemp); temp_assigned := false;

    erase(dcfile);
    rename(dctemp,dcfilename)
  end;

  procedure do_multi(defdrive: char);
  var
    disk: diskrec;
    drive: char;
    code: word;
    vol: namexstr;
    c1,c2: char;
    ok,changedisk: boolean;
  begin
    drive := defdrive;
    repeat
      txtinp(dcfile,dcfilename);
      txtout(dctemp,dctempname); temp_assigned := true;

      settextbuf(dcfile,fbuf^,stdbufsize);
      settextbuf(dctemp,obuf^,stdbufsize);

      changedisk := false;
      repeat
        if (not changedisk) then begin
          readdisk(drive,disk,code);
          ok := (code=0) and (disk.vol<>'')
        end else ok := false;
        if (not ok) then begin
          changedisk := false;
          writeln(stdout);
          write(stdout,'Please insert a labelled diskette in A: or B: [A:! B: Quit] ');
          clearkeys;
          repeat c1 := upcase(readkey)
          until (c1 in ['A','B','Q',#27,#13,' ']);
          case c1 of
            'A','B': begin writeln(stdout,'Drive ',c1,':'); drive := c1 end;
            #13,' ': writeln(stdout,'Drive ',drive,':');
            'Q',#27: writeln(stdout,'Quit')
          end;
          writeln(stdout)
        end
      until (ok) or (c1 in ['Q','q',#27]);

      if (ok) then begin
        changedisk := true;

        update(dcfile,dctemp,disk,code);
        if (code=1) then err('Cannot open '+dcfilename+' for reading');
        if (code=2) then err('Cannot open '+dctempname+' for writing');

        close(dcfile);
        close(dctemp); temp_assigned := false;

        erase(dcfile);
        rename(dctemp,dcfilename);

        writeln(stdout);
        write(stdout,'Another disk [Yes! No Quit A: B:] ');
        clearkeys;
        repeat c2 := upcase(readkey)
        until (c2 in ['Y','N','Q','A','B',#27,#13,' ']);
        case c2 of
          'Y',#13,' ': writeln(stdout,'Yep (Drive ',drive,':)');
          'A','B'    : begin writeln(stdout,'Yep (Drive ',c2,':)'); drive := c2 end;
          'N','Q',#27: writeln(stdout,'Nope');
        end;
        writeln(stdout)

      end
    until (c1 in ['Q',#27]) or (c2 in ['Q','N',#27]);

    if (temp_assigned) then begin
      erase(dctemp);
      if (ioresult<>0) then ;
      temp_assigned := false
    end
  end;

  procedure do_find(s: string);
  var
    code: word;
  begin
    txtinp(dcfile,dcfilename);

    settextbuf(dcfile,fbuf^,stdbufsize);

    findstr(dcfile,s,code);
    if (code=1) then err('Cannot open '+dcfilename+' for reading');

    close(dcfile)
  end;

  procedure do_remove(vol: namexstr);
  var
    code: word;
  begin
    txtinp(dcfile,dcfilename);
    txtout(dctemp,dctempname); temp_assigned := true;

    settextbuf(dcfile,fbuf^,stdbufsize);
    settextbuf(dctemp,obuf^,stdbufsize);

    remove(dcfile,dctemp,vol,code);
    if (code=1) then err('Cannot open '+dcfilename+' for reading');
    if (code=2) then err('Cannot open '+dctempname+' for writing');

    close(dcfile);
    close(dctemp); temp_assigned := false;

    erase(dcfile);
    rename(dctemp,dcfilename)
  end;

  procedure do_voldir(vol: namexstr);
  var
    code: word;
  begin
    txtinp(dcfile,dcfilename);

    settextbuf(dcfile,fbuf^,stdbufsize);

    voldir(dcfile,vol,code);
    if (code=1) then err('Cannot open '+dcfilename+' for reading');

    close(dcfile)
  end;

  procedure do_cat(vol: namexstr);
  var
    code: word;
  begin
    txtinp(dcfile,dcfilename);

    settextbuf(dcfile,fbuf^,stdbufsize);

    cat(dcfile,vol,code);
    if (code=1) then err('Cannot open '+dcfilename+' for reading');

    close(dcfile)
  end;

  procedure do_sum;
  var
    code: word;
  begin
    txtinp(dcfile,dcfilename);

    settextbuf(dcfile,fbuf^,stdbufsize);

    summarize(dcfile,code);
    if (code=1) then err('Cannot open '+dcfilename+' for reading');

    close(dcfile)
  end;

  { ---------------------------------------------------------------------- }
  {                          Selecting an action                           }
  { ---------------------------------------------------------------------- }

  procedure doit;
  var
    action: actiontype;
    drive: string[2];
  begin
    action := find_command(getname(1));
    case action of

      a_none,a_removx: err('Illegal Command "'+getname(1)+'"');
      a_ambig: err('Command "'+getname(1)+'" is not unique');

      a_add,a_update: begin
        if (nnam=2) and (checkdrive(getname(2))) then begin
          drive := getname(2);
          do_add(drive[1])
        end else if (nnam>1) then err('Too many parameters')
        else do_add(default_drive)
      end;

      a_multi: begin
        if (nnam=2) and (checkdrive(getname(2))) then begin
          drive := getname(2);
          do_multi(drive[1])
        end else if (nnam>1) then err('Too many parameters')
        else do_multi(default_drive)
      end;

      a_find: begin
        if (nnam=2) and (getname(2)<>'') then do_find(getname(2))
        else err('Search string must be specified')
      end;

      a_remove: begin
        if (nnam=2) and (getname(2)<>'') then do_remove(getname(2))
        else err('Volume must be specified')
      end;

      a_dir,a_list: begin
        if (nnam>2) then err('Too many parameters');
        if (nnam=1) then do_voldir('')
                    else do_voldir(getname(2))
      end;

      a_cat,a_volumes: begin
        if (nnam>2) then err('Too many parameters');
        if (nnam=1) then do_cat('')
                    else do_cat(getname(2))
      end;

      a_sum: begin
        if (nnam<>1) then err('Too many parameters')
        else do_sum
      end;

      a_help: help1
    end
  end;

{ ---------------------------------------------------------------------- }
{                          The main program                              }
{ ---------------------------------------------------------------------- }

begin
  temp_assigned := false;

  pre_op;
  proc_op;
  post_op;

  if (not optf) then begin
    dcfilename := progpath+progname+'.DAT';
    dctempname := tmpname(progpath)
  end else begin
    dctempname := tmpname(filedir(dcfilename))
  end;

  if (debug) then begin
    writeln(conout,'       Catalog file: ',dcfilename);
    writeln(conout,'       Temporary file: ',dctempname)
  end;

  { if the catalog file does not exist, create it }
  if (not exists(dcfilename)) then begin
    writeln(stdout,'The catalog ',dcfilename,' does not exist!');
    write(stdout,'Ok to create a new catalog file ? [Yes No!] ');
    repeat c := upcase(readkey)
    until (c='Y') or (c='N') or (c=#13) or (c=' ') or (c=#27);
    case c of
      'Y': begin
        Writeln(stdout,'Yep');
        assign(dcfile,dcfilename); rewrite(dcfile);
        if (ioresult<>0) then err('Cannot create '+dcfilename);
        close(dcfile)
      end;
      else begin Writeln(stdout,'Nope'); stop(0) end;
    end
  end;

  getmemory(fbuf,stdbufsize);
  getmemory(obuf,stdbufsize);

  doit;

  stop(0)
end.
@


1.6
log
@update to new email address
@
text
@d2 1
a2 1
{ $Id: dcat.pas 1.5 1993/03/18 11:47:22 Ricki Exp $ }
d32 3
d74 1
a74 1
  pversion = '$Id: dcat.pas 1.5 1993/03/18 11:47:22 Ricki Exp $';
d867 3
d908 4
@


1.5
log
@correction: some writes did not use the stdout file
@
text
@d2 1
a2 1
{ $Id: dcat.pas 1.4 1993/02/20 21:13:22 Ricki Exp $ }
d21 1
a21 1
  Email:  ricki@@rwthi3.informatik.rwth-aachen.de
d32 3
d71 1
a71 1
  pversion = '$Id: dcat.pas 1.4 1993/02/20 21:13:22 Ricki Exp $';
@


1.4
log
@bugfix: archive comment was overwritten by non-existing
  DESCRIPT.ION comment
@
text
@d2 1
a2 1
{ $Id: dcat.pas 1.3 1993/02/10 22:13:25 Ricki Exp $ }
d32 4
d68 1
a68 1
  pversion = '$Id: dcat.pas 1.3 1993/02/10 22:13:25 Ricki Exp $';
d1266 2
a1267 2
    writeln('The catalog ',dcfilename,' does not exist!');
    write('Ok to create a new catalog file ? [Yes No!] ');
d1272 1
a1272 1
        Writeln('Yep');
d1277 1
a1277 1
      else begin Writeln('Nope'); stop(0) end;
@


1.3
log
@modified -v page
@
text
@d2 1
a2 1
{ $Id: DCAT.PAS 1.2 1993/02/07 16:49:57 Ricki Exp $ }
d31 4
a34 1
        $Log: DCAT.PAS $
d64 1
a64 1
  pversion = '$Id: DCAT.PAS 1.2 1993/02/07 16:49:57 Ricki Exp $';
d814 1
a814 1
    c: string;
d851 2
a852 1
        getdesc(dfile,sr.name,c)
@


1.2
log
@new command alias: 'dir' = 'list'
avoided call of 'fileinfo'
@
text
@d2 1
a2 1
{ $Id: dcat.pas 1.1 1993/02/03 17:01:36 Ricki Exp $ }
d31 5
a35 1
        $Log: dcat.pas $
d61 1
a61 1
  pversion = '$Id: dcat.pas 1.1 1993/02/03 17:01:36 Ricki Exp $';
d65 2
a66 1
  desn = 'DCAT/2';
d127 1
a127 1
    header(year,func);
d134 1
a134 1
    header(year,func);
d141 1
a141 1
    header(year,func);
d143 1
a143 1
    revision(desn,'$Revision: 1.1 $');
d1261 1
a1261 1
    until (c='Y') or (c='N');
d1269 1
a1269 1
      'N': begin Writeln('Nope'); stop(0) end;
@


1.1
log
@Initial revision
@
text
@d2 1
a2 1
{ $Id: dcat.pas 1.1 1993/02/03 16:57:32 Ricki Exp $ }
d31 3
a33 1
        $Log$
d57 1
a57 1
  pversion = '$Id: dcat.pas 1.1 1993/02/03 16:57:32 Ricki Exp $';
a71 3
  cdel = #9; { separates pathnames from comments }
  lcdel = 1;

d88 1
a88 1
  actiontype = (a_none,a_ambig,
d90 2
a91 1
		a_removx,a_dir,a_sum,a_help);
d157 1
a157 1
	       or (s='a') or (s='a:') or (s='b') or (s='b:')
a807 1
    code: word;
d855 3
a857 6
        fileinfo(p+sr.name,size,time,attr,code);
        if (code<>0) then begin
          size := 0;
          time := 0;
          attr := 0
        end
d967 4
a970 2
      (#0,#0,'ADD','UPDATE','MULTI','CAT','VOLUMES',
             'FIND','REMOVE','REMOVx','DIR','SUM','HELP');
d976 1
a976 1
    for i:=a_add to a_help do begin
d1207 1
a1207 1
      a_dir: begin
@

