head	1.3;
access;
symbols
	rutils40:1.2;
locks;
comment	@        @;


1.3
date	93.03.31.19.23.11;	author Ricki;	state Exp;
branches;
next	1.2;

1.2
date	93.02.12.20.19.36;	author Ricki;	state Rel;
branches;
next	1.1;

1.1
date	93.01.21.21.40.55;	author Ricki;	state Exp;
branches;
next	;


desc
@Duplicate stdout to a file
@

1.3
log
@update to new email address
@
text
@{$I SWITCHES.INC}
{ $Id: tee.pas 1.2 1993/02/12 20:19:36 Ricki Exp $ }
program TEE;
{ Duplicate stdout to a file }

{
  This code is in the public domain. You may use it freely.
  There are no guarantees, either expressed or implied,
  as to either merchantability or fitness for a particular
  purpose. The author's liability is limited to the amount
  you paid for it.

  Richard Breuer

  Brunssumstrasse 6
  5100 Aachen
  Germany

  Phone:  +49/241/85605
  Fax:    +49/241/8021329
  Email:  ricki@@pool.informatik.rwth-aachen.de
}

{
        $Log: tee.pas $
        Revision 1.2  1993/02/12  20:19:36  Ricki
        modified -v page
        bugfix: redirection works now
        introduced text and binary mode, as in CAT

        Revision 1.1  1993/01/21  21:40:55  Ricki
        Initial revision
}

{
        Conditional defines
          DEBUG     Enables debugging mode. Sets debug := true
}

uses
  options,
  files,
  strings,
  standard,
  cmdline,
  outstd,
  outcon,
  dostools,
  dos;

const
  pversion = '$Id: tee.pas 1.2 1993/02/12 20:19:36 Ricki Exp $';

  func = 'Duplicate output to a file';
  year = '1992,1993';
  desn = 'TEE/2';
  desv = '1.0';
  opts = 'acChHv*'; ropt = opts;

var
  debug: boolean;
  fbuf: pointer;

  o: file;
  ot: text;
  asciimode,first: boolean;
  opt_a,opt_c,opt_cc: boolean;
  ofname: pathstr;

  {$i tee.i1}
  {$i tee.i2}

  {$i-} { turn off I/O checking for all procedures }

  procedure help1;
  begin
    header(desv,year,func);
    teei1;
    stop(1)
  end;

  procedure help2;
  begin
    header(desv,year,func);
    teei2;
    stop(1)
  end;

  procedure version;
  begin
    header(desv,year,func);
    writeln(stdout);
    revision(desn,desv);
    writeln(stdout);
    author;
    stop(1)
  end;

  {$f+} procedure pre_op(nopt: byte); {$f-}
  begin
    opt_a := false; opt_c := false; opt_cc := false;
{$IFDEF DEBUG}
    debug := true
{$ELSE}
    debug := false
{$ENDIF}
  end;

  {$f+} procedure proc_op(opt: char; opt_arg: namestring); {$f-}
  begin
    case opt of
      'a': opt_a := true;
      'c': opt_c := true;
      'C': opt_cc := true;

      #0 : help1;
      'h': help1;
      'H': help2;
      'v': version;
      '*': debug := true;
    end
  end;

  {$f+} procedure post_op; {$f-}
  begin
    if (debug) then startdbg;
    report(ropt,pversion);

    { determine the operation mode }
    if (opt_c) then asciimode := false
    	       else asciimode := (opt_cc) or (not outredirected)
  end;

  {$f+} procedure pre_fn(nnam: byte); {$f-}
  var
    fm: byte;
  begin
    first := true; { first is true for the first name on the command line }

    { there must be at least 1 input parameter }
    if (nnam=0) then err('Missing outfile');

    { check the first name, whether it is one single file }
    ofname := strupcase(getname(1));
    if (ofname='') or (maybemany(ofname)) then
      err('Illegal output file - no wildcards allowed');

    { check for appending / overwriting }
    if (opt_a) then begin
      { set filemode to "reset = read & write" }
      fm := filemode; filemode := 2;
      if (asciimode) then begin
        assign(ot,ofname); reset(ot)
      end else begin
        assign(o,ofname); reset(o,1)
      end;
      filemode := fm;

      { if file does not exist, create it }
      if (ioresult<>0) then begin
        if (asciimode) then rewrite(ot)
	               else rewrite(o,1)
      end
      { else if it exists, place the pointer at the end }
      else begin
        if (asciimode) then append(ot)
	               else seek(o,filesize(o)-1)
      end

    end else begin
      if (asciimode) then begin
        assign(ot,ofname); rewrite(ot)
      end else begin
        assign(o,ofname); rewrite(o,1)
      end
    end;
    if (ioresult<>0) then err('Cannot open "'+ofname+'" for writing');

    { get mem for a buffer }
    getmem(fbuf,stdbufsize)
  end;

  {$f+} procedure proc_fn(var s: searchrec; p: pathstr); {$f-}
  var
    tf: text;      { ascii }
    line: string;  {       }
    f,stdo: file;  { non-ascii }
    rlr,rlw: word; {           }
  begin
    { do NOT process the first name, since it's the name of the output file }
    if (not first) then begin
      { distinguish between ascii and nonascii mode }
      if (not asciimode) then begin
        bininp(f,p+s.name);
        binout(stdo,'');

        if (debug) then begin
          if (s.name<>'') then writeln(conout,'       Bin-Reading ',p+s.name)
                          else writeln(conout,'       Bin-Reading stdin');
        end;

        { copy f to stdo and o }
        rlr := stdbufsize;
        while (not eof(f)) and (rlr=stdbufsize) do begin
          blockread(f,fbuf^,stdbufsize,rlr);

          if (debug) then
            writeln(conout,'       Read  ',rlr,
                           '/',stdbufsize,' bytes from ',p+s.name);

          if (rlr>0) then begin
            blockwrite(stdo,fbuf^,rlr,rlw);

            if (debug) then
              writeln(conout,'       Wrote ',rlw,
                             '/',rlr,' bytes to stdout');

	    if (rlr<>rlw) then
              err('Error writing to stdout');

            blockwrite(o,fbuf^,rlr,rlw);

            if (debug) then
              writeln(conout,'       Wrote ',rlw,
                             '/',rlr,' bytes to '+ofname);

	    if (rlr<>rlw) then
              err('Error writing to '+ofname);
          end
        end;

        close(f);
        close(stdo)
      end else begin
        txtinp(tf,p+s.name);
        settextbuf(tf,fbuf^,stdbufsize);
        if (debug) then begin
          if (s.name<>'') then writeln(conout,'       Asc-Reading ',p+s.name)
                          else writeln(conout,'       Asc-Reading stdin');
        end;

        while (not eof(tf)) do begin
          readln(tf,line);
          writeln(stdout,line);
          writeln(ot,line)
        end;

        close(tf)
      end
    end else first := false
  end;

  {$f+} procedure post_fn; {$f-}
  var
    s: searchrec;
    nopt,nnam: byte;
    procedure assign_stdin(var sr: searchrec);
    { fills sr with the appropriate values for stdin }
    begin
      sr.name := '';
      sr.attr := archive;
      sr.size := 0;
      sr.time := 0;
    end;
  begin
    { special case: nnam=1 and input redirected. In this case proc_fn did
      nothing. We read stdin to get some data }
    parse(ropt,nopt,nnam);
    if (nnam=1) and (inpredirected) then begin
      assign_stdin(s);
      first := false;
      proc_fn(s,'')
    end;
    if (asciimode) then close(ot) else close(o);
    freemem(fbuf,stdbufsize)
  end;

begin
  process(ropt,
          pre_op,proc_op,post_op,
          pre_fn,proc_fn,post_fn,
          m_stdin or m_pipe);
  stop(0)
end.
@


1.2
log
@modified -v page
bugfix: redirection works now
introduced text and binary mode, as in CAT
@
text
@d2 1
a2 1
{ $Id: TEE.PAS 1.1 1993/01/21 21:40:55 Ricki Exp $ }
d21 1
a21 1
  Email:  ricki@@rwthi3.informatik.rwth-aachen.de
d25 6
a30 1
        $Log: TEE.PAS $
d52 1
a52 1
  pversion = '$Id: TEE.PAS 1.1 1993/01/21 21:40:55 Ricki Exp $';
@


1.1
log
@Initial revision
@
text
@d1 2
a2 1
{ $Id: TEE.PAS 1.1 1993/01/21 21:22:32 Ricki Exp $ }
d26 2
a27 3
 * Revision 1.1  1993/01/21  21:22:32  Ricki
 * Initial revision
 *
d43 1
d47 1
a47 1
  pversion = '$Id: TEE.PAS 1.1 1993/01/21 21:22:32 Ricki Exp $';
d52 2
a53 1
  opts = 'ahHv*'; ropt = opts;
d60 3
a62 3
  first: boolean;
  opta: boolean;
  nnam: byte;
d68 2
d72 1
a72 1
    header(year,func);
d79 1
a79 1
    header(year,func);
d86 1
a86 1
    header(year,func);
d88 1
a88 1
    revision(desn,'$Revision: 1.1 $');
d96 1
a96 1
    opta := false;
d104 1
a104 1
  {$f+} procedure proc_op(opt: char; optarg: namestring); {$f-}
d107 3
a109 1
      'a': opta := true;
d123 4
d135 2
a136 3
    { there must be at least 2 input parameters }
    if (nnam<2) then
      err('Not enough parameters');
d141 1
a141 1
      err('Illegal output file');
d143 2
a144 2
    { open output file }
    if (opta) then begin
d147 5
a151 1
      {$i-} assign(o,ofname); reset(o,1); {$i+}
d156 2
a157 1
        {$i-} assign(o,ofname); rewrite(o,1) {$i+}
d160 4
a163 1
      else seek(o,filesize(o)-1)
d166 5
a170 1
      {$i-} assign(o,ofname); rewrite(o,1) {$i+}
d180 4
a183 2
    f,stdo: file;
    rlr,rlw: word;
d187 14
a200 19
      bininp(f,p+s.name);
      binout(stdo,'');

      if (debug) then begin
        if (s.name<>'') then writeln(conout,'       Reading ',p+s.name)
                        else writeln(conout,'       Reading stdin');
      end;

      { append the contents of f to o }
      rlr := stdbufsize;
      while (not eof(f)) and (rlr=stdbufsize) do begin
        blockread(f,fbuf^,stdbufsize,rlr);

        if (debug) then
          writeln(conout,'       Read  ',rlr,
                         '/',stdbufsize,' bytes from ',p+s.name);

        if (rlr>0) then begin
          blockwrite(o,fbuf^,rlr,rlw);
d203 2
a204 2
            writeln(conout,'       Wrote ',rlw,
                           '/',rlr,' bytes to ',ofname);
d206 2
a207 2
	  if (rlr<>rlw) then
            err('Error writing to output file (disk full?)');
d209 33
a241 1
          blockwrite(stdo,fbuf^,rlr,rlw);
d243 2
a244 11
          if (debug) then
            writeln(conout,'       Wrote ',rlw,
                           '/',rlr,' bytes to stdout');

	  if (rlr<>rlw) then
            err('Error writing to stdout');
        end
      end;

      close(f);
      close(stdo)
d249 11
d261 9
a269 1
    close(o);
@


1.1
log
@Initial revision
@
text
@{ $Id$ }
program TEE;
{ Duplicate stdout to a file }

{
  This code is in the public domain. You may use it freely.
  There are no guarantees, either expressed or implied,
  as to either merchantability or fitness for a particular
  purpose. The author's liability is limited to the amount
  you paid for it.

  Richard Breuer

  Brunssumstrasse 6
  5100 Aachen
  Germany

  Phone:  +49/241/85605
  Fax:    +49/241/8021329
  Email:  ricki@@rwthi3.informatik.rwth-aachen.de
}

{
        $Log$
}

{
        Conditional defines
          DEBUG     Enables debugging mode. Sets debug := true
}

uses
  options,
  files,
  strings,
  standard,
  cmdline,
  outstd,
  outcon,
  dos;

const
  pversion = '$Id$';

  func = 'Duplicate output to a file';
  year = '1992,1993';
  desn = 'TEE/2';
  opts = 'ahHv*'; ropt = opts;

var
  debug: boolean;
  fbuf: pointer;

  o: file;
  first: boolean;
  opta: boolean;
  nnam: byte;
  ofname: pathstr;

  {$i tee.i1}
  {$i tee.i2}

  procedure help1;
  begin
    header(year,func);
    teei1;
    stop(1)
  end;

  procedure help2;
  begin
    header(year,func);
    teei2;
    stop(1)
  end;

  procedure version;
  begin
    header(year,func);
    writeln(stdout);
    revision(desn,'$Revision: 4.5 $');
    writeln(stdout);
    author;
    stop(1)
  end;

  {$f+} procedure pre_op(nopt: byte); {$f-}
  begin
    opta := false;
{$IFDEF DEBUG}
    debug := true
{$ELSE}
    debug := false
{$ENDIF}
  end;

  {$f+} procedure proc_op(opt: char; optarg: namestring); {$f-}
  begin
    case opt of
      'a': opta := true;

      #0 : help1;
      'h': help1;
      'H': help2;
      'v': version;
      '*': debug := true;
    end
  end;

  {$f+} procedure post_op; {$f-}
  begin
    if (debug) then startdbg;
    report(ropt,pversion);
  end;

  {$f+} procedure pre_fn(nnam: byte); {$f-}
  var
    fm: byte;
  begin
    first := true; { first is true for the first name on the command line }

    { there must be at least 2 input parameters }
    if (nnam<2) then
      err('Not enough parameters');

    { check the first name, whether it is one single file }
    ofname := strupcase(getname(1));
    if (ofname='') or (maybemany(ofname)) then
      err('Illegal output file');

    { open output file }
    if (opta) then begin
      { set filemode to "reset = read & write" }
      fm := filemode; filemode := 2;
      {$i-} assign(o,ofname); reset(o,1); {$i+}
      filemode := fm;

      { if file does not exist, create it }
      if (ioresult<>0) then begin
        {$i-} assign(o,ofname); rewrite(o,1) {$i+}
      end
      { else if it exists, place the pointer at the end }
      else seek(o,filesize(o)-1)

    end else begin
      {$i-} assign(o,ofname); rewrite(o,1) {$i+}
    end;
    if (ioresult<>0) then err('Cannot open "'+ofname+'" for writing');

    { get mem for a buffer }
    getmem(fbuf,stdbufsize)
  end;

  {$f+} procedure proc_fn(var s: searchrec; p: pathstr); {$f-}
  var
    f,stdo: file;
    rlr,rlw: word;
  begin
    { do NOT process the first name, since it's the name of the output file }
    if (not first) then begin
      bininp(f,p+s.name);
      binout(stdo,'');

      if (debug) then begin
        if (s.name<>'') then writeln(conout,'       Reading ',p+s.name)
                        else writeln(conout,'       Reading stdin');
      end;

      { append the contents of f to o }
      rlr := stdbufsize;
      while (not eof(f)) and (rlr=stdbufsize) do begin
        blockread(f,fbuf^,stdbufsize,rlr);

        if (debug) then
          writeln(conout,'       Read  ',rlr,
                         '/',stdbufsize,' bytes from ',p+s.name);

        if (rlr>0) then begin
          blockwrite(o,fbuf^,rlr,rlw);

          if (debug) then
            writeln(conout,'       Wrote ',rlw,
                           '/',rlr,' bytes to ',ofname);

	  if (rlr<>rlw) then
            err('Error writing to output file (disk full?)');

          blockwrite(stdo,fbuf^,rlr,rlw);

          if (debug) then
            writeln(conout,'       Wrote ',rlw,
                           '/',rlr,' bytes to stdout');

	  if (rlr<>rlw) then
            err('Error writing to stdout');
        end
      end;

      close(f);
      close(stdo)
    end else first := false
  end;

  {$f+} procedure post_fn; {$f-}
  begin
    close(o);
    freemem(fbuf,stdbufsize)
  end;

begin
  process(ropt,
          pre_op,proc_op,post_op,
          pre_fn,proc_fn,post_fn,
          m_stdin or m_pipe);
  stop(0)
end.
@

