{ $Id$ }
UNIT dBASE3p;
{ read and write access to dBase III+ .DBF files }

{$R-}

INTERFACE

  TYPE
    DbfFieldType = RECORD
      FdName   : String[10];
      FdType   : Char;		{ C N L D M }
      FdLength : Byte;
      FdDec    : Byte;
    END;

    DbfFieldTypeA = ARRAY [1..1] OF DbfFieldType;

    DbfFileType = RECORD
      dirty         : Boolean; { true iff modified }

      VersionNumber : Byte;
      Update        : ARRAY [1..3] OF Byte;
      NbrRec        : Longint;
      HdrLen        : Integer;
      RecLen        : Word;
      NbrFlds       : Integer;
      FileSize      : Longint;
      FileHndl      : FILE;
      FileName      : String[12];
      FieldStru     : ^DbfFieldTypeA;
    END;

    DbfFile = ^DbfFileType;

  FUNCTION DbfOpen(FN: String; var d: DbfFile): boolean;
  FUNCTION DbfClose(D: DbfFile): Boolean;
  FUNCTION DbfReadHdr(D: DbfFile): Boolean;
  FUNCTION DbfWriteHdr(D: DbfFile): Boolean;
  FUNCTION DbfReadStru(D: DbfFile): Boolean;
  FUNCTION DbfWriteStru(D: DbfFile): Boolean;
  function DbfReadRec(RecNum: Longint; D: DbfFile; var rec): boolean;
  function DbfWriteRec(RecNum: Longint; D: DbfFile; var rec): boolean;

IMPLEMENTATION

  uses
    dos;

  const
    c_hdrlen = 1+3+4+2+2+20+1; { length of the constant header part }

  type
    charArray = array [0..0] of char;

  FUNCTION DbfOpen(FN: String; var d: DbfFile): boolean;
  var
    fm: byte;
    dd,mm,yy,dw: word;
  BEGIN
    GetMem(D,SizeOf(DbfFileType));
    fm := system.filemode;
    system.filemode := 2;
{$IFDEF DEBUG}{$i-}{$ENDIF}
    Assign(D^.FileHndl, FN);
    Reset(D^.FileHndl,1);  {Set record length to 1}
{$IFDEF DEBUG}{$i+}{$ENDIF}
    dbfopen := (ioresult=0);
    system.filemode := fm;

    getdate(yy,mm,dd,dw);
    with d^ do begin
      dirty         := false;

      VersionNumber := 3;
      Update[1]     := yy-1900;
      Update[2]     := mm;
      Update[3]     := dd;
      NbrRec	    := 0;
      HdrLen	    := c_hdrlen;
      RecLen	    := 0;
      NbrFlds	    := 0;
      FileSize      := c_hdrlen;
      FileName	    := fn;
      FieldStru	    := nil
    end
  END;

  FUNCTION DbfClose(D: DbfFile): Boolean;
  BEGIN
    if (d^.Dirty) then begin
      if (not DbfWriteHdr(D)) then exit
    end;
    Close(D^.FileHndl);
    FreeMem(D^.FieldStru,
               SizeOf(DbfFieldType)*(D^.NbrFlds+1));
    FreeMem(D,SizeOf(DbfFileType));
    DbfClose := TRUE
  END;

  FUNCTION DbfReadHdr(D: DbfFile): Boolean;
  TYPE
     DbfHdrMask = RECORD
        VersionNumber : Byte;
        Update        : ARRAY [1..3] OF Byte;
        NbrRec        : Longint;
        HdrLen        : Integer;
        RecLen        : Integer;
        Reserved      : ARRAY [1..20] OF Char;
     END;
  VAR
    Result : Word;
    H : DbfHdrMask;
    I : Byte;
  BEGIN
    dbfreadhdr := false;
{$IFDEF DEBUG}{$i-}{$ENDIF}
    BlockRead(D^.FileHndl, H, SizeOf(H), Result);
{$IFDEF DEBUG}{$i+}{$ENDIF}
    IF SizeOf(H) = Result THEN WITH D^ DO BEGIN
      VersionNumber := H.VersionNumber;
      FOR I := 1 TO 3 DO Update[I] := H.Update[I];
      NbrRec := H.NbrRec;
      HdrLen := H.HdrLen;
      RecLen := H.RecLen;
      NbrFlds := (H.HdrLen - 33) DIV 32;
      FileSize := H.HdrLen + H.RecLen * H.NbrRec + 1;
      IF (VersionNumber and 7<>3) THEN exit; {Not a dBase file }
    END ELSE exit; {Error reading Dbf}
    dbfreadhdr := true;
    d^.dirty := false
  END; {FUNCTION}

  function DbfWriteHdr(D: DbfFile): boolean;
  TYPE
     DbfHdrMask = RECORD
        VersionNumber : Byte;
        Update        : ARRAY [1..3] OF Byte;
        NbrRec        : Longint;
        HdrLen        : Integer;
        RecLen        : Integer;
        Reserved      : ARRAY [1..20] OF Char;
     END;
  VAR
    Result : Word;
    H : DbfHdrMask;
    I : Byte;
  BEGIN with d^ do begin
    h.VersionNumber := VersionNumber;
    FOR I := 1 TO 3 DO h.Update[I] := Update[I];
    h.NbrRec := NbrRec;
    h.HdrLen := HdrLen;
    h.RecLen := RecLen;
    h.hdrlen := c_hdrlen+NbrFlds*32;
    seek(FileHndl,0);
{$IFDEF DEBUG}{$i-}{$ENDIF}
    blockwrite(filehndl,h,sizeof(h),result);
{$IFDEF DEBUG}{$i+}{$ENDIF}
    dbfwritehdr := (ioresult=0) and (result=sizeof(h));
    dirty := false
  end END;

  FUNCTION DbfReadStru(D: DbfFile): Boolean;
  TYPE
     DbfFieldMask = RECORD
        FdName    : ARRAY [1..11] OF Char;
        FdType    : Char;
        Reserved1 : ARRAY [1..4] OF Char;
        FdLength  : Byte;
        FdDec     : Byte;
        Reserved2 : ARRAY [1..14] OF Char;
     END;
  VAR
    Result : Word;
    I, J, HdrTerminator : Byte;
    FldTmp : DbfFieldMask;
    hdrlen: longint;
  BEGIN
    dbfreadstru := false;
    GetMem(D^.FieldStru,SizeOf(DbfFieldType)*D^.NbrFlds);
    hdrlen := c_hdrlen+d^.NbrFlds*32;
    seek(d^.filehndl,hdrlen);
    FOR I := 1 TO D^.NbrFlds DO BEGIN
{$IFDEF DEBUG}{$i-}{$ENDIF}
      BlockRead(D^.FileHndl,FldTmp,SizeOf(FldTmp),Result);
{$IFDEF DEBUG}{$i+}{$ENDIF}
      if (ioresult<>0) or (result<>sizeof(fldtmp)) then exit;
      WITH DbfFieldType(D^.FieldStru^[I]) DO BEGIN
        J := POS(#0,FldTmp.FdName);
        if (j=0) then j := 12;
        FdName := Copy(FldTmp.FdName,1,J-1);
        FdType   := FldTmp.FdType;
        FdLength := FldTmp.FdLength;
        FdDec    := FldTmp.FdDec
      END
    END;
    {Last Hdr Byte}
{$IFDEF DEBUG}{$i-}{$ENDIF}
    BlockRead(D^.FileHndl,HdrTerminator,1,Result);
{$IFDEF DEBUG}{$i+}{$ENDIF}
    dbfreadstru := (HdrTerminator=13)
               and (result=sizeof(HdrTerminator))
               and (ioresult=0)
  END;

  FUNCTION DbfwriteStru(D: DbfFile): Boolean;
  const
    hdrterminator: char = #13;
  TYPE
     DbfFieldMask = RECORD
        FdName    : ARRAY [1..11] OF Char;
        FdType    : Char;
        Reserved1 : ARRAY [1..4] OF Char;
        FdLength  : Byte;
        FdDec     : Byte;
        Reserved2 : ARRAY [1..14] OF Char;
     END;
  VAR
    Result : Word;
    I, J: Byte;
    FldTmp : DbfFieldMask;
    hdrlen: longint;
  BEGIN
    dbfwritestru := false;
    hdrlen := c_hdrlen+d^.NbrFlds*32;
    seek(d^.filehndl,hdrlen);
    FOR I := 1 TO D^.NbrFlds DO BEGIN
      WITH DbfFieldType(D^.FieldStru^[I]) DO BEGIN
        for j:=1 to length(fdname) do fldtmp.fdname[j] := fdname[j];
        for j:=length(fdname) to 11 do fldtmp.fdname[j] := #0;
        FldTmp.FdType   := FdType;
        FldTmp.FdLength := FdLength;
        FldTmp.FdDec    := FdDec
      END;
{$IFDEF DEBUG}{$i-}{$ENDIF}
      blockwrite(d^.filehndl,fldtmp,sizeof(fldtmp),result);
{$IFDEF DEBUG}{$i+}{$ENDIF}
      if (ioresult<>0) or (result<>sizeof(fldtmp)) then exit
    END;
    {Last Hdr Byte}
{$IFDEF DEBUG}{$i-}{$ENDIF}
    Blockwrite(D^.FileHndl,HdrTerminator,1,Result);
{$IFDEF DEBUG}{$i+}{$ENDIF}
    dbfwritestru := (result=sizeof(HdrTerminator))
                and (ioresult=0)
  END;

  function DbfReadRec(RecNum : Longint; D: DbfFile; var rec): boolean;
  VAR
    DbfRec	: CharArray absolute rec;
    Result      : Word;
    CurrentPos  : Longint;
  BEGIN
    DbfReadRec := false;
    if (RecNum<1) or (RecNum>d^.nbrRec) then exit;
    CurrentPos := (RecNum-1) * D^.RecLen+D^.HdrLen;
{$IFDEF DEBUG}{$i-}{$ENDIF}
    Seek(D^.FileHndl,CurrentPos);
    BlockRead(D^.FileHndl,DbfRec,D^.RecLen,Result);
{$IFDEF DEBUG}{$i+}{$ENDIF}
    dbfreadrec := (result=d^.reclen)
    	      and (ioresult=0)
  END;

  function DbfWriteRec(RecNum : Longint; D: DbfFile; var rec): boolean;
  VAR
    DbfRec	: CharArray absolute rec;
    Result      : Word;
    CurrentPos  : Longint;
  BEGIN
    DbfWriteRec := false;
    if (RecNum<1) or (RecNum>1+d^.nbrRec) then exit;
    { account for appends }
    if (RecNum>d^.nbrRec) then begin
      d^.dirty := true;
      inc(d^.nbrrec);
      inc(d^.filesize,d^.reclen)
    end;
    CurrentPos := (RecNum-1) * D^.RecLen+D^.HdrLen;
{$IFDEF DEBUG}{$i-}{$ENDIF}
    Seek(D^.FileHndl,CurrentPos);
    BlockWrite(D^.FileHndl,DbfRec,D^.RecLen,Result);
{$IFDEF DEBUG}{$i+}{$ENDIF}
    dbfwriterec := (result=d^.reclen)
    	       and (ioresult=0)
  END;

END.
