{ $Id$ }
UNIT dBASE3p;
{ read and write access to dBase III+ .DBF files }

{$R-}

INTERFACE

  uses
    dos;

  TYPE
    DBFieldType = RECORD
      Nam: String[10];
      Typ: Char;	{ C N L D M }
      Len: Byte;
      Dec: Byte;
    END;

    DBFieldTypeA = ARRAY [1..1] OF DBFieldType;
    DBFieldTypeP = ^DBFieldTypeA;
    charArray = array [0..0] of char;

    DBFileType = RECORD
      VersionNumber: Byte;
      Update       : ARRAY [1..3] OF Byte;
      NbrRec       : Longint;
      HdrLen       : Integer;
      RecLen       : Word;
      NbrFlds      : Integer;
      FileSize     : Longint;
      FileHndl     : FILE;
      FieldStru    : DBFieldTypeP;

      buf: ^CharArray;	{ internal buffer }
      dirty: boolean	{ internal flag }
    END;

    DBFile = ^DBFileType;

  function  DBopen(fn: string; var db: dbfile): boolean;
  function  DBcreate(fn: string; var db: dbfile): boolean;
  function  DBclose(var db: dbfile): boolean;
  function  DBaddfield(var db: dbfile; fld: DBfieldtype): boolean;
  function  DBread(var db: dbfile; no: longint; var rec; var del: boolean): boolean;
  function  DBwrite(var db: dbfile; no: longint; var rec): boolean;
  function  DBdelete(var db: dbfile; no: longint): boolean;
  function  DBundelete(var db: dbfile; no: longint): boolean;
  function  DBsize(db: dbfile): longint;
  function  DBfilesize(db: dbfile): longint;
  procedure DBfields(db: dbfile; var n: integer; var fld: DBFieldTypeP);
  {
    DBopen	opens a DBF file for reading and reads header and
  		structure part into db
    DBcreate	opens a DBF file for writing and creates an empty
  		header and structure part
    DBaddfield	adds a field to an _empty_ DBF file
    DBread	reads a record; you must provide a string entry of an
    		appropriate length for each entry, eg.
                	record
                          name   : string[30];
                          address: string[40];
                          iq     : string[10];	 /* a number field */
                        end
    DBwrite	writes a record, which must be input in the above format
    DBdelete  	marks a record as deleted
    DBundelete	marks a record as undeleted
    DBsize	returns the number of records in a database
    DBfilesize	returns the number of bytes a database occupies
    DBfields    returns the number of fields in a database and a pointer to
    		an array which contains the field information; the contents
		of the address the pointer points to may not be modified!
  }

implementation

  const
    c_hdrlen = 1+3+4+2+2+20+1; { length of the constant header part }

  FUNCTION DBOpen(FN: String; var db: DBFile): boolean;
  var
    i,j,fm,HdrTerminator: byte;
    result: word;
    H: RECORD
      VersionNumber: Byte;
      Update       : ARRAY [1..3] OF Byte;
      NbrRec       : Longint;
      HdrLen       : Integer;
      RecLen       : Integer;
      Reserved     : ARRAY [1..20] OF Char;
    END;
    fldtmp: RECORD
      FdName   : ARRAY [1..11] OF Char;
      FdType   : Char;
      Reserved1: ARRAY [1..4] OF Char;
      FdLength : Byte;
      FdDec    : Byte;
      Reserved2: ARRAY [1..14] OF Char;
    END;
  BEGIN
    DBopen := false;
    GetMem(Db,SizeOf(DBFileType));

    { open the file }
    fm := system.filemode;
    system.filemode := 2;
{$IFDEF DEBUG}{$i-}{$ENDIF}
    Assign(Db^.FileHndl,FN);
    Reset(Db^.FileHndl,1);  {Set record length to 1}
{$IFDEF DEBUG}{$i+}{$ENDIF}
    system.filemode := fm;
    if (ioresult<>0) then exit;

    { read the header }
{$IFDEF DEBUG}{$i-}{$ENDIF}
    BlockRead(Db^.FileHndl, H, SizeOf(H), Result);
{$IFDEF DEBUG}{$i+}{$ENDIF}
    if (Result<>sizeof(H)) or (ioresult<>0) then exit;
    WITH Db^ DO BEGIN
      VersionNumber := H.VersionNumber;
      FOR I := 1 TO 3 DO Update[I] := H.Update[I];
      NbrRec := H.NbrRec;
      HdrLen := H.HdrLen;
      RecLen := H.RecLen;
      getmem(buf,reclen);	{ mem for reading one record }
      NbrFlds := (H.HdrLen - 33) DIV 32;
      FileSize := H.HdrLen + H.RecLen * H.NbrRec + 1;
      IF (VersionNumber and 7<>3) THEN exit; {Not a dBase file }
    END;

    { read the structure }
    GetMem(Db^.FieldStru,SizeOf(DBFieldType)*Db^.NbrFlds);
    FOR I := 1 TO Db^.NbrFlds DO BEGIN
{$IFDEF DEBUG}{$i-}{$ENDIF}
      BlockRead(Db^.FileHndl,FldTmp,SizeOf(FldTmp),Result);
{$IFDEF DEBUG}{$i+}{$ENDIF}
      if (ioresult<>0) or (result<>sizeof(fldtmp)) then exit;
      WITH DBFieldType(Db^.FieldStru^[I]) DO BEGIN
        J := POS(#0,FldTmp.FdName);
        if (j=0) then j := 12;
        Nam := Copy(FldTmp.FdName,1,J-1);
        Typ := FldTmp.FdType;
        Len := FldTmp.FdLength;
        Dec := FldTmp.FdDec
      END
    END;
    {Last Hdr Byte}
{$IFDEF DEBUG}{$i-}{$ENDIF}
    BlockRead(Db^.FileHndl,HdrTerminator,1,Result);
{$IFDEF DEBUG}{$i+}{$ENDIF}
    if (HdrTerminator<>13)
      or (result<>sizeof(HdrTerminator))
      or (ioresult<>0) then exit;

    db^.dirty := false;
    dbopen := true
  END;

  function DBcreate(fn: string; var db: dbfile): boolean;
  var
    fm: byte;
    dd,mm,yy,dw: word;
  BEGIN
    DBcreate := false;
    GetMem(Db,SizeOf(DBFileType));

    { open file for writing }
    fm := system.filemode;
    system.filemode := 2;
{$IFDEF DEBUG}{$i-}{$ENDIF}
    Assign(Db^.FileHndl, FN);
    Rewrite(Db^.FileHndl,1);  {Set record length to 1}
{$IFDEF DEBUG}{$i+}{$ENDIF}
    system.filemode := fm;
    if (ioresult<>0) then exit;

    { preset header & structure values }
    getdate(yy,mm,dd,dw);
    with db^ do begin
      VersionNumber := 3;
      Update[1]     := yy-1900;
      Update[2]     := mm;
      Update[3]     := dd;
      NbrRec	    := 0;
      HdrLen	    := c_hdrlen;
      RecLen	    := 1;
      buf := nil;
      NbrFlds	    := 0;
      FileSize      := c_hdrlen+1;
      FieldStru	    := nil
    end;

    db^.dirty := true;
    dbcreate := true
  END;

  FUNCTION DBClose(var Db: DBFile): Boolean;
  const
    hdrterminator: char = #13;
    fileterminator: char = #26;
  var
    Result: Word;
    I,j: Byte;
    h: RECORD
      VersionNumber: Byte;
      Update       : ARRAY [1..3] OF Byte;
      NbrRec       : Longint;
      HdrLen       : Integer;
      RecLen       : Integer;
      Reserved     : ARRAY [1..20] OF Char;
    END;
    fldtmp: RECORD
      FdName    : ARRAY [1..11] OF Char;
      FdType    : Char;
      Reserved1 : ARRAY [1..4] OF Char;
      FdLength  : Byte;
      FdDec     : Byte;
      Reserved2 : ARRAY [1..14] OF Char;
    END;
  BEGIN
    dbclose := false;

    { write header, if modified since opening time }
    if (db^.dirty) then with db^ do begin
      { write header }
      h.VersionNumber := VersionNumber;
      FOR I := 1 TO 3 DO h.Update[I] := Update[I];
      h.NbrRec := NbrRec;
      h.HdrLen := HdrLen;
      h.RecLen := RecLen;
      h.hdrlen := c_hdrlen+NbrFlds*32;
      h.reserved := #0#0#0#0#0#0#0#0#0#0#0#0#0#0#0#0#0#0#0#0;
{$IFDEF DEBUG}{$i-}{$ENDIF}
      seek(FileHndl,0);
      if (ioresult<>0) then exit;
      blockwrite(filehndl,h,sizeof(h),result);
{$IFDEF DEBUG}{$i+}{$ENDIF}
      if (ioresult<>0) or (result<>sizeof(h)) then exit;

      { write structure }
      FOR I := 1 TO NbrFlds DO BEGIN
        WITH DbFieldType(FieldStru^[I]) DO BEGIN
          for j:=1 to length(nam) do fldtmp.fdname[j] := nam[j];
          for j:=length(nam)+1 to 11 do fldtmp.fdname[j] := #0;
          FldTmp.FdType    := Typ;
          FldTmp.FdLength  := Len;
          FldTmp.FdDec     := Dec;
          fldtmp.reserved1 := #0#0#0#0;
          fldtmp.reserved2 := #0#0#0#0#0#0#0#0#0#0#0#0#0#0;
        END;
{$IFDEF DEBUG}{$i-}{$ENDIF}
        blockwrite(filehndl,fldtmp,sizeof(fldtmp),result);
{$IFDEF DEBUG}{$i+}{$ENDIF}
        if (ioresult<>0) or (result<>sizeof(fldtmp)) then exit
      END;
      {Last Hdr Byte}
{$IFDEF DEBUG}{$i-}{$ENDIF}
      Blockwrite(FileHndl,HdrTerminator,1,Result);
{$IFDEF DEBUG}{$i+}{$ENDIF}
      if (result<>sizeof(HdrTerminator)) or (ioresult<>0) then exit;

      { end the file with ^Z (#26) if it was modified }
{$IFDEF DEBUG}{$i-}{$ENDIF}
      seek(FileHndl,filesize-1);
      if (ioresult<>0) then exit;
      blockwrite(filehndl,fileterminator,sizeof(fileterminator),result);
{$IFDEF DEBUG}{$i+}{$ENDIF}
      if (ioresult<>0) or (result<>sizeof(fileterminator)) then exit;
    end;

{$IFDEF DEBUG}{$i-}{$ENDIF}
    Close(Db^.FileHndl);
{$IFDEF DEBUG}{$i+}{$ENDIF}
    if (ioresult<>0) then exit;
    FreeMem(Db^.FieldStru,SizeOf(DBFieldType)*Db^.NbrFlds);
    FreeMem(db^.buf,db^.reclen);
    FreeMem(Db,SizeOf(DBFileType));
    DBClose := true
  END;

  function DBaddfield(var db: dbfile; fld: DBfieldtype): boolean;
  var
    h: dbfieldtypeP;
    j: byte;
  begin
    { check if the file is empty }
    if (db^.nbrrec<>0) then begin
      dbaddfield := false;
      exit
    end else dbaddfield := true;

    { append the field to the header }
    GetMem(h,SizeOf(DBFieldType)*(Db^.NbrFlds+1));
    move(db^.fieldstru^,h^,SizeOf(DBFieldType)*Db^.NbrFlds);
    h^[Db^.NbrFlds+1] := fld;
    FreeMem(db^.fieldstru,SizeOf(DBFieldType)*Db^.NbrFlds);
    db^.fieldstru := h;
    inc(Db^.NbrFlds);
    inc(Db^.HdrLen,32);
    inc(Db^.filesize,32);
    if (db^.nbrflds>1) then freemem(db^.buf,db^.reclen);
    inc(db^.reclen,fld.len);
    getmem(db^.buf,db^.reclen);

    db^.dirty := true
  end;

  function DBread(var db: dbfile; no: longint; var rec; var del: boolean): boolean;
  VAR
    r: chararray absolute rec;
    p,Result: Word;
    i: byte;
    CurrentPos: Longint;
  BEGIN
    DBRead := false;
    if (no<1) or (no>db^.nbrRec) then exit;
    CurrentPos := (no-1)*Db^.RecLen+Db^.HdrLen;
{$IFDEF DEBUG}{$i-}{$ENDIF}
    Seek(Db^.FileHndl,CurrentPos);
    if (ioresult<>0) then exit;
    BlockRead(Db^.FileHndl,db^.buf^,Db^.RecLen,Result);
{$IFDEF DEBUG}{$i+}{$ENDIF}
    if (result<>db^.reclen) or (ioresult<>0) then exit;

    { check if the record is deleted }
    del := (db^.buf^[0]<>' ');

    { transfer the buffer into a 'readable' format }
    p := 0;
    for i:=1 to db^.nbrflds do begin
      r[p+i-1] := chr(db^.fieldstru^[i].len);
      move(db^.buf^[p+1],r[p+i],db^.fieldstru^[i].len);
      inc(p,db^.fieldstru^[i].len)
    end;

    dbread := true
  END;

  function DBwrite(var db: dbfile; no: longint; var rec): boolean;
  VAR
    i: byte;
    r: chararray absolute rec;
    p,Result: Word;
    CurrentPos: Longint;
  BEGIN
    DBWrite := false;
    if (no<1) or (no>1+db^.nbrRec) or (db^.nbrflds=0) then exit;

    { account for appends }
    if (no>db^.nbrRec) then begin
      inc(db^.nbrrec);
      inc(db^.filesize,db^.reclen);
      db^.dirty := true
    end;

    { transfer the data from the 'readable' into the internal format }
    p := 0;
    for i:=1 to db^.nbrflds do begin
      move(r[p+i],db^.buf^[p+1],db^.fieldstru^[i].len);
      inc(p,db^.fieldstru^[i].len)
    end;

    { write record }
    CurrentPos := (no-1)*Db^.RecLen+Db^.HdrLen;
{$IFDEF DEBUG}{$i-}{$ENDIF}
    Seek(Db^.FileHndl,CurrentPos);
    if (ioresult<>0) then exit;
    BlockWrite(Db^.FileHndl,db^.buf^,Db^.RecLen,Result);
{$IFDEF DEBUG}{$i+}{$ENDIF}
    dbwrite := (result=db^.reclen) and (ioresult=0)
  END;

  {local} function setmark(var db: dbfile; no: longint; mark: char): boolean;
  var
    Result: Word;
    currentpos: longint;
  begin
    setmark := false;
    if (no<1) or (no>db^.nbrRec) then exit;

    CurrentPos := (no-1)*Db^.RecLen+Db^.HdrLen;
{$IFDEF DEBUG}{$i-}{$ENDIF}
    Seek(Db^.FileHndl,CurrentPos);
    if (ioresult<>0) then exit;
    BlockWrite(Db^.FileHndl,mark,1,Result);
{$IFDEF DEBUG}{$i+}{$ENDIF}
    setmark := (result=1) and (ioresult=0)
  end;

  function DBdelete(var db: dbfile; no: longint): boolean;
  begin DBdelete := setmark(db,no,'*') end;

  function DBundelete(var db: dbfile; no: longint): boolean;
  begin DBundelete := setmark(db,no,' ') end;

  function DBsize(db: dbfile): longint;
  begin DBsize := db^.nbrrec end;

  function DBfilesize(db: dbfile): longint;
  begin DBfilesize := db^.filesize end;

  procedure DBfields(db: dbfile; var n: integer; var fld: DBFieldTypeP);
  begin n := db^.nbrflds; fld := db^.fieldstru end;

end.
