{$I SWITCHES.INC}
{ $Id: dbutl.pas 1.2 1993/02/11 13:35:33 Ricki Exp $ }
program dbutl;
{ Display structure and contents of dBase III+ DBF files }

{
  This code is in the public domain. You may use it freely.
  There are no guarantees, either expressed or implied,
  as to either merchantability or fitness for a particular
  purpose. The author's liability is limited to the amount
  you paid for it.

  Richard Breuer

  Brunssumstrasse 6
  5100 Aachen
  Germany

  Phone:  +49/241/85605
  Fax:    +49/241/8021329
  Email:  ricki@pool.informatik.rwth-aachen.de
}

{
        $Log$
}

{
        Conditional defines
          DEBUG     Enables debugging mode. Sets debug := true
}

uses
  dbase3p,
  strings,
  standard,
  outcon,
  cmdline,
  outstd,
  dos;

const
  pversion = '$Id: dbutl.pas 1.2 1993/02/11 13:35:33 Ricki Exp $';

  func = 'Display structure and contents of dBase III+ DBF files';
  year = '1992,1993';
  desn = 'DBUTL/2';
  desv = '1.0';
  opts = 'hHlrsv*'; ropt = opts;

var
  fbuf: pointer;
  debug: boolean;
  opt_r,opt_s,opt_l: boolean;

  {$i dbutl.i1}
  {$i dbutl.i2}

  procedure help1;
  begin
    header(desv,year,func);
    dbutli1;
    stop(1)
  end;

  procedure help2;
  begin
    header(desv,year,func);
    dbutli2;
    stop(1)
  end;

  procedure version;
  begin
    header(desv,year,func);
    writeln(stdout);
    revision(desn,desv);
    writeln(stdout);
    author;
    stop(1)
  end;

  {$f+} procedure pre_op(nopt: byte); {$f-}
  begin
    opt_r := false; opt_s := false; opt_l := false;
{$IFDEF DEBUG}
    debug := true
{$ELSE}
    debug := false
{$ENDIF}
  end;

  {$f+} procedure proc_op(opt: char; optarg: namestring); {$f-}
  begin
    case opt of
      'r': opt_r := true;
      's': opt_s := true;
      'l': opt_l := true;

      #0 : help1;
      'h': help1;
      'H': help2;
      'v': version;
      '*': debug := true;
    end
  end;

  {$f+} procedure post_op; {$f-}
  begin
    if (debug) then startdbg;
    report(ropt,pversion);

    if (not (opt_r or opt_s or opt_l)) then begin
      opt_r := true;
      opt_s := true
    end
  end;

  {$f+} procedure pre_fn(nnam: byte); {$f-}
  begin
  end;

  {$f+} procedure proc_fn(var s: searchrec; p: pathstr); {$f-}
  var
    db: dbfile;
    fields: dbfieldtypep;
    nfields,i,tot: integer;
    r: longint;
    buf: ^CharArray;
    str: string;
    del: boolean;
  begin
    if (not DBopen(p+s.name,db)) then err('Cannot open "'+p+s.name+'"');
    if (debug) then begin
      if (s.name<>'') then writeln(conout,'       Reading ',p+s.name)
                      else writeln(conout,'       Reading stdin');
    end;

    writeln(stdout,copy(s.name+' =========================',1,26));

    { record number }
    if (opt_r) then begin
      writeln(stdout);
      writeln(stdout,'Total: ',DBsize(db),' Records');
    end;

    { file structure }
    if (opt_s) then begin
      writeln(stdout);
      writeln(stdout,'File Structure -----------');
      DBfields(db,nfields,fields);
{$IFDEF DEBUG}{$r-}{$ENDIF}
      tot := 0;
      for i:=1 to nfields do with fields^[i] do begin
        if (dec<>0) then
	  writeln(stdout,copy(nam+blanks,1,15),len:3,' ',dec:3,'   ',typ)
        else
	  writeln(stdout,copy(nam+blanks,1,15),len:3,'       ',typ);
        inc(tot,len)
      end;
{$IFDEF DEBUG}{$r+}{$ENDIF}
      writeln(stdout,'Total:      ',tot:6,'   Bytes')
    end;

    if (opt_l) then begin
      writeln(stdout);
      DBfields(db,nfields,fields);
      { allocate as much memory as necessary for the data }
      getmem(buf,tot+nfields);
{$IFDEF DEBUG}{$r-}{$ENDIF}
      for r:=1 to DBsize(db) do if (DBread(db,r,buf^,del)) then begin
        if (del) then writeln(stdout,'-deleted- ',r)
      		 else writeln(stdout,'--------- ',r);
        tot := 0;
        for i:=1 to nfields do with fields^[i] do begin
	  write(stdout,copy(nam+blanks,1,13));
          move(buf^[tot],str[0],1+len);
          writeln(stdout,str);
          inc(tot,len+1)
        end
      end;
{$IFDEF DEBUG}{$r+}{$ENDIF}
      freemem(buf,tot+nfields)
    end;

    if (not DBclose(db)) then ;
  end;

  {$f+} procedure post_fn; {$f-}
  begin
  end;

begin
  process(ropt,
          pre_op,proc_op,post_op,
          pre_fn,proc_fn,post_fn,
          m_none);
  stop(0)
end.
